__author__ = 'mahajrod'
import os

from itertools import cycle, islice
from collections import Iterable
#import matplotlib
#matplotlib.use('Agg')
os.environ['MPLCONFIGDIR'] = '/tmp/'
import matplotlib.pyplot as plt
plt.ioff()
from matplotlib.transforms import Bbox, TransformedBbox, blended_transform_factory
from mpl_toolkits.axes_grid1.inset_locator import BboxPatch, BboxConnector, BboxConnectorPatch
from matplotlib.lines import Line2D
from matplotlib.colors import LogNorm

from matplotlib_venn import venn2, venn3

import numpy as np

from RouToolPa.Collections.General import IdList, SynDict, IdSet


class MatplotlibRoutines:
    def __init__(self):
        pass

    @staticmethod
    def connect_bbox(bbox1, bbox2,
                     loc1a, loc2a, loc1b, loc2b,
                     prop_lines, prop_patches=None):
        if prop_patches is None:
            prop_patches = prop_lines.copy()
            prop_patches["alpha"] = prop_patches.get("alpha", 1)*0.2

        c1 = BboxConnector(bbox1, bbox2, loc1=loc1a, loc2=loc2a, **prop_lines)
        c1.set_clip_on(False)
        c2 = BboxConnector(bbox1, bbox2, loc1=loc1b, loc2=loc2b, **prop_lines)
        c2.set_clip_on(False)

        bbox_patch1 = BboxPatch(bbox1, **prop_patches)
        bbox_patch2 = BboxPatch(bbox2, **prop_patches)

        p = BboxConnectorPatch(bbox1, bbox2,
                               loc1a=loc1a, loc2a=loc2a, loc1b=loc1b, loc2b=loc2b,
                               **prop_patches)
        p.set_clip_on(False)

        return c1, c2, bbox_patch1, bbox_patch2, p

    def zoom_effect(self, ax1, ax2, xmin, xmax, alpha=0.22, color="gray", **kwargs):
        """
        ax1 : the main axes
        ax2 : the zoomed axes
        (xmin,xmax) : the limits of the colored area in both plot axes.

        connect ax1 & ax2. The x-range of (xmin, xmax) in both axes will
        be marked.  The keywords parameters will be used ti create
        patches.

        """

        trans1 = blended_transform_factory(ax1.transData, ax1.transAxes)
        trans2 = blended_transform_factory(ax2.transData, ax2.transAxes)

        bbox = Bbox.from_extents(xmin, 0, xmax, 1)

        mybbox1 = TransformedBbox(bbox, trans1)
        mybbox2 = TransformedBbox(bbox, trans2)

        prop_patches = kwargs.copy()
        prop_patches["ec"] = "none"
        prop_patches["alpha"] = alpha
        prop_patches["color"] = color

        c1, c2, bbox_patch1, bbox_patch2, p = self.connect_bbox(mybbox1, mybbox2,
                                                                loc1a=3, loc2a=2, loc1b=4, loc2b=1,
                                                                prop_lines=kwargs, prop_patches=prop_patches)

        ax1.add_patch(bbox_patch1)
        ax2.add_patch(bbox_patch2)
        ax2.add_patch(c1)
        ax2.add_patch(c2)
        ax2.add_patch(p)

        return c1, c2, bbox_patch1, bbox_patch2, p

    @staticmethod
    def annotated_heatmap(data, row_labels, column_labels, subplot=None, colorbar_kw={}, colorbar_label="",
                          title=None, xlabel=None, ylabel=None,
                          horizontal_ticks_angle=0, **kwargs):
        """
        Create a heatmap from a numpy array and two lists of labels.

        Arguments:
            data       : A 2D numpy array of shape (N,M)
            row_labels : A list or array of length N with the labels
                         for the rows
            col_labels : A list or array of length M with the labels
                         for the columns
        Optional arguments:
            ax         : A matplotlib.axes.Axes instance to which the heatmap
                         is plotted. If not provided, use current axes or
                         create a new one.
            cbar_kw    : A dictionary with arguments to
                         :meth:`matplotlib.Figure.colorbar`.
            cbarlabel  : The label for the colorbar
        All other arguments are directly passed on to the imshow call.
        """

        if not subplot:
            subplot = plt.gca()

        # Plot the heatmap
        image = subplot.imshow(data, **kwargs)

        # Create colorbar
        colorbar = subplot.figure.colorbar(image, ax=subplot, **colorbar_kw)
        colorbar.ax.set_ylabel(colorbar_label, rotation=-90, va="bottom")

        # We want to show all ticks...
        subplot.set_xticks(np.arange(data.shape[1]))
        subplot.set_yticks(np.arange(data.shape[0]))
        # ... and label them with the respective list entries.
        subplot.set_xticklabels(column_labels)
        subplot.set_yticklabels(row_labels)

        # Let the horizontal axes labeling appear on top.
        #subplot.tick_params(top=True, bottom=False, labeltop=True, labelbottom=False)

        # Rotate the tick labels and set their alignment.
        plt.setp(subplot.get_xticklabels(), rotation=horizontal_ticks_angle, ha="right",
                 rotation_mode="anchor")

        # Turn spines off and create white grid.
        for edge, spine in subplot.spines.items():
            spine.set_visible(False)

        subplot.set_xticks(np.arange(data.shape[1]+1)-.5, minor=True)
        subplot.set_yticks(np.arange(data.shape[0]+1)-.5, minor=True)
        subplot.grid(which="minor", color="w", linestyle='-', linewidth=3)
        subplot.tick_params(which="minor", bottom=False, left=False)

        if title is not None:
            plt.title(title)
        if xlabel is not None:
            plt.xlabel(xlabel)
        if ylabel is not None:
            plt.ylabel(ylabel)

        return image, colorbar

    @staticmethod
    def percent_histogram(data, output_prefix=None, n_bins=20, title="", xlabel="%", ylabel="Number", label=None,
                          extensions=("png", "svg"), legend=None, legend_location="best", input_mode="percent", xmax=None,
                          xmin=None, stats_as_legend=True):
        if output_prefix:
            figure = plt.figure()
            subplot = plt.subplot(1, 1, 1)

        if isinstance(n_bins, Iterable):
            number_of_bins = n_bins
        else:
            if input_mode == "percent":
                number_of_bins = np.linspace(0, 100, n_bins+1)
            elif input_mode == "fraction":
                number_of_bins = np.linspace(0, 1.0, n_bins+1)

        n, bins, patches = plt.hist(data, bins=number_of_bins, label=label)

        #print n
        #print bins
        #print patches
        if input_mode == "percent":
            plt.xlim(xmin=0, xmax=100)
        elif input_mode == "fraction":
            plt.xlim(xmin=0, xmax=1)
        else:
            raise ValueError("Unrecognized type of input data(neither percents nor fractions)")

        if title:
            plt.title(title)
        if xlabel:
            plt.xlabel(xlabel)
        if ylabel:
            plt.ylabel(ylabel)

        if stats_as_legend:
            legenda = "Total: %i\nMean: %.2f %%\nMedian: %.2f %%" if input_mode == "percent" else "Total: %i\nMean: %.2f\nMedian: %.2f"
            legenda = legenda % (len(data), np.mean(data), np.median(data)) if stats_as_legend else legend
            plt.legend((legenda,), loc=legend_location)
        else:
            if legend:
                plt.legend((legend,), loc=legend_location)
        if label:
            plt.legend(loc=legend_location)

        if output_prefix:
            for ext in extensions:
                plt.savefig("%s.%s" % (output_prefix, ext))

    def percent_histogram_from_file(self, data_file, output_prefix, data_type=float, column_list=None, separator=None,
                                    comments="#", n_bins=20, title="", xlabel="%", ylabel="Number",
                                    extensions=("png", "svg"), legend=None, legend_location="best",
                                    stats_as_legend=False, input_mode="percent", label=None):
        #print column_list
        data = np.loadtxt(data_file, dtype=data_type, comments=comments, delimiter=separator, usecols=column_list)
        if input_mode == "percent":
            n_bins = np.linspace(0, 100, n_bins+1)
        elif input_mode == "fraction":
            n_bins = np.linspace(0, 1.0, n_bins+1)
        else:
            raise ValueError("Unrecognized type of input data(neither percents nor fractions)")
        #legenda = "Total: %i\nMean: %.2f %%\nMedian: %.2f %%" if input_mode == "percent" else "Total: %i\nMean: %.2f\nMedian: %.2f"
        #legenda = legenda % (len(data), np.mean(data), np.median(data)) if stats_as_legend else legend
        
        self.percent_histogram(data, output_prefix=output_prefix, n_bins=n_bins, title=title, xlabel=xlabel,
                               ylabel=ylabel, extensions=extensions, legend=legend, legend_location=legend_location,
                               input_mode=input_mode, label=label, stats_as_legend=stats_as_legend)

    @staticmethod
    def add_line(axes, start, end, color):
        line = Line2D([start[0], end[0]], [start[1], end[1]], color=color)
        return axes.add_line(line)

    @staticmethod
    def int_histogram(data, output_prefix, n_bins=None, title="", xlabel="%", ylabel="Number",
                      extensions=("png", "svg"), legend=None, legend_location="best"):



        figure = plt.figure()
        subplot = plt.subplot(1, 1, 1)

        maximum = np.max(data)
        minimum = np.min(data)
        number_of_bins = n_bins if n_bins else maximum - minimum + 1
        plt.hist(data, bins=number_of_bins)

        plt.xlim(xmin=minimum - 1, xmax=maximum)

        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        if legend:
            plt.legend((legend,), loc=legend_location)
        for ext in extensions:
            plt.savefig("%s.%s" % (output_prefix, ext))

    @staticmethod
    def extended_percent_histogram(data, output_prefix, n_bins=20, title="", xlabel="%", ylabel="Number", label=None,
                                   extensions=("png", "svg"), legend=None, legend_location="best", input_mode="percent",
                                   xmax=None, xmin=None):

        figure = plt.figure(figsize=(16, 16))


        maximum = np.max(data)

        bins = np.linspace(0, 100, 21)
        bins = np.append(bins, maximum + 1)

        histogram_list = [np.histogram(dataset, bins=bins)[0] for dataset in data]
        bins = bins[:-1]
        #print bins
        #print histogram_list

        number_of_histograms = len(histogram_list)

        width = 5.0 / float((number_of_histograms + 2))
        #print width

        color_cycle = cycle(['b', 'r', 'c', 'g', 'indigo', 'y', 'k', 'olive', 'violet', 'darkgrey', 'gold'])

        color_list = list(islice(color_cycle, None, number_of_histograms))

        for subplot_index in (1, 2):
            subplot = plt.subplot(2, 1, subplot_index)
            for i in range(0, number_of_histograms):
                left = bins + (i + 1) * width
                #print left
                #print histogram_list[i]
                subplot.bar(left, histogram_list[i], width, label=list(label)[i], color=color_list[i],)

            plt.xlim(xmin=0, xmax=105)

            plt.title(title)
            plt.xlabel(xlabel)
            plt.ylabel(ylabel)

            if subplot_index == 1:
                subplot.set_yscale('log', basey=10)
            elif subplot_index == 2:
                if legend:
                    plt.legend((legend,), loc=legend_location)
                if label:
                    plt.legend(loc=legend_location)
        for ext in extensions:
            plt.savefig("%s.%s" % (output_prefix, ext))

    @staticmethod
    def draw_histogram(data_array, output_prefix=None, number_of_bins=None, width_of_bins=None, bins_list=None,
                       max_threshold=None, min_threshold=None, xlabel=None, ylabel=None,
                       title=None, extensions=("png",), ylogbase=None, xlogbase=None, subplot=None, suptitle=None,
                       close_figure=False, save_histovalues_only=False, figsize=(6, 6),
                       header=None):
        if (number_of_bins is not None) and (width_of_bins is not None):
            raise AttributeError("Options -w/--width_of_bins and -b/--number_of_bins mustn't be set simultaneously")

        if max_threshold and min_threshold:
            if max_threshold < min_threshold:
                raise ValueError("Maximum threshold (%s) is lower than minimum threshold(%s)" % (str(max_threshold),
                                                                                                 str(min_threshold)))

        max_lenn = max(data_array)
        min_lenn = min(data_array)

        max_len = max_threshold if (max_threshold is not None) else max_lenn
        min_len = min_threshold if (min_threshold is not None) else min_lenn


        #max_len = max_threshold if (max_threshold is not None) and (max_threshold < max_lenn) else max_lenn
        #min_len = min_threshold if (min_threshold is not None) and (min_lenn < min_threshold) else min_lenn

        filtered = []

        if (max_len < max_lenn) and (min_len > min_lenn):
            for entry in data_array:
                if min_len <= entry <= max_len:
                    filtered.append(entry)
        elif max_len < max_lenn:
            for entry in data_array:
                if entry <= max_len:
                    filtered.append(entry)
        elif min_len > min_lenn:
            for entry in data_array:
                if min_len <= entry:
                    filtered.append(entry)
        else:
            filtered = data_array
        if subplot is None:
            #print "aaaaaaaaaa"
            figure = plt.figure(1, figsize=figsize,)
            subplot = figure.add_subplot(1, 1, 1)
        else:
            plt.axes(subplot)

        if bins_list:
            bins = bins_list
        elif number_of_bins:
            bins = number_of_bins
        elif width_of_bins:
            bins = np.arange(min_len, max_len, width_of_bins)
            #print bins
            #bins[0] += 1
            bins = np.append(bins, [max_len])
        else:
            bins = 30

        n, bins, patches = plt.hist(filtered, bins=bins) # , log=False if ylogbase is None else True)

        bin_centers = (bins + ((bins[1] - bins[0])/2))[:-1]
        #print bin_centers
        #print len(n)
        #print len(bin_centers)
        print ("Minimum x in histogram: %f" % float(min_len))
        print ("Maximum x in histogram: %f" % float(max_len))
        print ("Bins:")
        print (bins)

        plt.xlim(xmin=min_len, xmax=max_len)
        if xlabel:
            plt.xlabel(xlabel)
        if ylabel:
            plt.ylabel(ylabel)
        if title:
            plt.title(title)
        if suptitle:
            plt.suptitle(suptitle)

        if ylogbase:
            subplot.set_yscale('log', basey=ylogbase)
            plt.ylim(ymin=1)

        if xlogbase:
            subplot.set_xscale('log', basex=xlogbase)
            plt.xlim(xmin=1)

        if output_prefix:
            if not save_histovalues_only:
                for ext in extensions:
                    plt.savefig("%s.%s" % (output_prefix, ext))

            # save histo values
            np.savetxt("%s.histo" % output_prefix, zip(bins[:-1], n), fmt="%i\t%i")
            np.savetxt("%s.bins" % output_prefix, bins, fmt="%i")
        if close_figure:
            plt.close(figure)

        return n, bins

        #return zip(bin_centers, n)

    def draw_multi_histogram_picture(self, list_of_data_arrays, subplot_tuple, output_prefix=None,
                                     figsize=(10, 10), number_of_bins_list=None, width_of_bins_list=None,
                                     max_threshold_list=None, min_threshold_list=None, xlabel_list=None, ylabel_list=None,
                                     title_list=None, ylogbase_list=None, label_list=None,
                                     extensions=("png",), suptitle=None, share_y_axis=False,
                                     share_x_axis=False):
        figure = plt.figure(1, figsize=figsize)
        if suptitle:
            plt.suptitle(suptitle)
        if len(subplot_tuple) != 2:
            raise ValueError("Subplot tuple should contain exactly two values, not %i!" % len(subplot_tuple))
        if not (isinstance(subplot_tuple[0], int) and isinstance(subplot_tuple[1], int)):
            raise ValueError("Subplot tuple should contain two values, not (%s, %s)!" % (str(type(subplot_tuple[0])),
                                                                                         str(type(subplot_tuple[1]))))

        number_of_subplots = subplot_tuple[0] * subplot_tuple[1]
        number_of_datasets = len(list_of_data_arrays)

        parameters_list = [number_of_bins_list, width_of_bins_list, max_threshold_list, min_threshold_list,
                           xlabel_list, ylabel_list, title_list, ylogbase_list, label_list]

        subplot_list = []
        for dataset_index in range(0, number_of_datasets):
            parameters = [None, None, None, None, None, None, None, None, None]
            for parameter_index in range(0, 9):
                if parameters_list[parameter_index]:
                    if dataset_index < len(parameters_list[parameter_index]):
                        parameters[parameter_index] = parameters_list[parameter_index][dataset_index]
            if dataset_index > 0:
                if share_x_axis and share_y_axis:
                    subplot_list.append(figure.add_subplot(subplot_tuple[0],
                                                           subplot_tuple[1],
                                                           dataset_index + 1,
                                                           sharex=subplot_list[0],
                                                           sharey=subplot_list[0]))
                elif share_x_axis:
                    subplot_list.append(figure.add_subplot(subplot_tuple[0],
                                                           subplot_tuple[1],
                                                           dataset_index + 1,
                                                           sharex=subplot_list[0]))
                elif share_y_axis:
                    subplot_list.append(figure.add_subplot(subplot_tuple[0],
                                                           subplot_tuple[1],
                                                           dataset_index + 1,
                                                           sharex=subplot_list[0],
                                                           sharey=subplot_list[0]))
                else:
                    subplot_list.append(figure.add_subplot(subplot_tuple[0],
                                                           subplot_tuple[1],
                                                           dataset_index + 1))
            else:
                subplot_list.append(figure.add_subplot(subplot_tuple[0],
                                                       subplot_tuple[1],
                                                       dataset_index + 1))

            histo = self.draw_histogram(list_of_data_arrays[dataset_index],  number_of_bins=parameters[0],
                                        width_of_bins=parameters[1], max_threshold=parameters[2],
                                        min_threshold=parameters[3], xlabel=parameters[4], ylabel=parameters[5],
                                        title=parameters[6], extensions=("png",), ylogbase=parameters[7],
                                        subplot=subplot_list[dataset_index],
                                        suptitle=None)
            #print histo
            if output_prefix:
                output_histo_file = "%s.%s.%shisto" % (output_prefix,
                                                       dataset_index if parameters[8] is None else parameters[9],
                                                       ("log%i." % parameters[7]) if parameters[7] else "")
                np.savetxt(output_histo_file, histo, fmt="%i\t%i")

        if output_prefix:
            for ext in extensions:
                plt.savefig("%s.%s" % (output_prefix, ext))

        return figure

    def draw_tetra_histogram_with_two_logscaled(self, list_of_data_arrays,  output_prefix=None,
                                                figsize=(10, 10), number_of_bins_list=None, width_of_bins_list=None,
                                                max_threshold_list=None, min_threshold_list=None, xlabel=None, ylabel=None,
                                                title_list=None, logbase=10, label_list=None,
                                                extensions=("png",), suptitle=None, share_y_axis=False,
                                                share_x_axis=False):
        subplot_tuple = (2, 2)

        none_list = [None, None, None, None]

        list_of_data = [list_of_data_arrays[0], list_of_data_arrays[1],
                        list_of_data_arrays[0], list_of_data_arrays[1]]

        number_of_bins_listtt = [number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0] if len(number_of_bins_list) == 1 else number_of_bins_list[1],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0] if len(number_of_bins_list) == 1 else number_of_bins_list[1]] if number_of_bins_list else none_list

        width_of_bins_listtt = [width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0] if len(width_of_bins_list) == 1 else width_of_bins_list[1],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0] if len(width_of_bins_list) == 1 else width_of_bins_list[1]] if width_of_bins_list else none_list

        max_threshold_listtt = [max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0] if len(max_threshold_list) == 1 else max_threshold_list[1],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0] if len(max_threshold_list) == 1 else max_threshold_list[1]] if max_threshold_list else none_list

        min_threshold_listtt = [min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0] if len(min_threshold_list) == 1 else min_threshold_list[1],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0] if len(min_threshold_list) == 1 else min_threshold_list[1]] if min_threshold_list else none_list

        title_listtt = [title_list[0], title_list[1],
                        "%s(logscaled)" % title_list[0], "%s(logscaled)" % title_list[1]] if title_list else none_list

        ylogbase_list = [None, None, logbase, logbase]

        xlabel_list = [None,
                       None,
                       xlabel if isinstance(xlabel, str) else xlabel[0] if len(xlabel) == 1 else xlabel[0],
                       xlabel if isinstance(xlabel, str) else xlabel[0] if len(xlabel) == 1 else xlabel[1]]

        ylabel_list = [ylabel if isinstance(ylabel, str) else ylabel[0] if len(ylabel) == 1 else ylabel[0],
                       None,
                       ylabel if isinstance(ylabel, str) else ylabel[0] if len(ylabel) == 1 else ylabel[1],
                       None]

        self.draw_multi_histogram_picture(list_of_data, subplot_tuple, output_prefix=output_prefix,
                                          figsize=figsize, number_of_bins_list=number_of_bins_listtt,
                                          width_of_bins_list=width_of_bins_listtt,
                                          max_threshold_list=max_threshold_listtt,
                                          min_threshold_list=min_threshold_listtt, xlabel_list=xlabel_list, ylabel_list=ylabel_list,
                                          title_list=title_listtt, ylogbase_list=ylogbase_list, label_list=None,
                                          extensions=extensions, suptitle=suptitle,
                                          share_y_axis=share_y_axis, share_x_axis=share_x_axis)

    def draw_hexa_histogram_with_three_logscaled(self, list_of_data_arrays,  output_prefix=None,
                                                 figsize=(10, 15), number_of_bins_list=None, width_of_bins_list=None,
                                                 max_threshold_list=None, min_threshold_list=None, xlabel=None, ylabel=None,
                                                 title_list=None, logbase=10, label_list=None,
                                                 extensions=("png",), suptitle=None, share_y_axis=False,
                                                 share_x_axis=False):
        subplot_tuple = (2, 3)

        none_list = [None, None, None, None, None, None]

        list_of_data = [list_of_data_arrays[0], list_of_data_arrays[1], list_of_data_arrays[2],
                        list_of_data_arrays[0], list_of_data_arrays[1], list_of_data_arrays[2]]

        number_of_bins_listtt = [number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0] if len(number_of_bins_list) == 1 else number_of_bins_list[1],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0] if len(number_of_bins_list) == 1 else number_of_bins_list[2],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0] if len(number_of_bins_list) == 1 else number_of_bins_list[1],
                                 number_of_bins_list if isinstance(number_of_bins_list, str) else number_of_bins_list[0] if len(number_of_bins_list) == 1 else number_of_bins_list[2],] if number_of_bins_list else none_list

        width_of_bins_listtt = [width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0] if len(width_of_bins_list) == 1 else width_of_bins_list[1],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0] if len(width_of_bins_list) == 1 else width_of_bins_list[2],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0] if len(width_of_bins_list) == 1 else width_of_bins_list[1],
                                width_of_bins_list if isinstance(width_of_bins_list, str) else width_of_bins_list[0] if len(width_of_bins_list) == 1 else width_of_bins_list[2]] if width_of_bins_list else none_list

        max_threshold_listtt = [max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0] if len(max_threshold_list) == 1 else max_threshold_list[1],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0] if len(max_threshold_list) == 1 else max_threshold_list[2],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0] if len(max_threshold_list) == 1 else max_threshold_list[1],
                                max_threshold_list if isinstance(max_threshold_list, str) else max_threshold_list[0] if len(max_threshold_list) == 1 else max_threshold_list[2]] if max_threshold_list else none_list

        min_threshold_listtt = [min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0] if len(min_threshold_list) == 1 else min_threshold_list[1],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0] if len(min_threshold_list) == 1 else min_threshold_list[2],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0] if len(min_threshold_list) == 1 else min_threshold_list[1],
                                min_threshold_list if isinstance(min_threshold_list, str) else min_threshold_list[0] if len(min_threshold_list) == 1 else min_threshold_list[2],] if min_threshold_list else none_list

        """


        number_of_bins_listtt = [number_of_bins_list[0], number_of_bins_list[1], number_of_bins_list[2],
                                 number_of_bins_list[0], number_of_bins_list[1], number_of_bins_list[2]] if number_of_bins_list else none_list
        width_of_bins_listtt = [width_of_bins_list[0], width_of_bins_list[1], width_of_bins_list[2],
                                width_of_bins_list[0], width_of_bins_list[1], width_of_bins_list[2]] if width_of_bins_list else none_list
        max_threshold_listtt = [max_threshold_list[0], max_threshold_list[1], max_threshold_list[2],
                                max_threshold_list[0], max_threshold_list[1], max_threshold_list[2]] if max_threshold_list else none_list

        min_threshold_listtt = [min_threshold_list[0], min_threshold_list[1], min_threshold_list[2],
                                min_threshold_list[0], min_threshold_list[1], min_threshold_list[2]] if min_threshold_list else none_list

        """

        title_listtt = [title_list[0], title_list[1], title_list[2],
                        "%s, logscaled" % title_list[0], "%s, logscaled" % title_list[1], "%s, logscaled" % title_list[2]] if title_list else none_list

        ylogbase_list = [None, None, None, logbase, logbase, logbase]

        xlabel_list = [None,
                       None,
                       None,
                       xlabel if isinstance(xlabel, str) else xlabel[0] if len(xlabel) == 1 else xlabel[0],
                       xlabel if isinstance(xlabel, str) else xlabel[0] if len(xlabel) == 1 else xlabel[1],
                       xlabel if isinstance(xlabel, str) else xlabel[0] if len(xlabel) == 1 else xlabel[2]]

        ylabel_list = [ylabel if isinstance(ylabel, str) else ylabel[0] if len(ylabel) == 1 else ylabel[0],
                       None,
                       None,
                       ylabel if isinstance(ylabel, str) else ylabel[0] if len(ylabel) == 1 else ylabel[1],
                       None,
                       None]

        """
        xlabel_list = [None,
                       None,
                       None,
                       xlabel if isinstance(xlabel, str) else xlabel[0],
                       xlabel if isinstance(xlabel, str) else xlabel[1],
                       xlabel if isinstance(xlabel, str) else xlabel[2]]
        ylabel_list = [ylabel if isinstance(ylabel, str) else ylabel[0],
                       None,
                       None,
                       ylabel if isinstance(ylabel, str) else ylabel[1],
                       None,
                       None]
        """

        self.draw_multi_histogram_picture(list_of_data, subplot_tuple, output_prefix=output_prefix,
                                          figsize=figsize, number_of_bins_list=number_of_bins_listtt,
                                          width_of_bins_list=width_of_bins_listtt,
                                          max_threshold_list=max_threshold_listtt,
                                          min_threshold_list=min_threshold_listtt, xlabel_list=xlabel_list, ylabel_list=ylabel_list,
                                          title_list=title_listtt, ylogbase_list=ylogbase_list, label_list=None,
                                          extensions=extensions, suptitle=suptitle, share_y_axis=share_y_axis,
                                          share_x_axis=share_x_axis)

    def draw_tetra_histogram_with_two_logscaled_from_file(self, file_list,  column_idx_list, output_prefix,
                                                          figsize=(10, 10), number_of_bins_list=None, width_of_bins_list=None,
                                                          max_threshold_list=None, min_threshold_list=None, xlabel=None, ylabel=None,
                                                          title_list=None, logbase=10, label_list=None,
                                                          extensions=("png",), suptitle=None, separator=None,
                                                          share_y_axis=False, share_x_axis=False,
                                                          comments="#"):

        list_of_data_arrays = []
        for filename, column_idx in zip(file_list, column_idx_list):
            #print filename, column_idx
            list_of_data_arrays.append(np.loadtxt(filename, usecols=(column_idx,),
                                                  delimiter=separator, comments=comments))

        self.draw_tetra_histogram_with_two_logscaled(list_of_data_arrays, output_prefix=output_prefix,
                                                     figsize=figsize, number_of_bins_list=number_of_bins_list,
                                                     width_of_bins_list=width_of_bins_list,
                                                     max_threshold_list=max_threshold_list,
                                                     min_threshold_list=min_threshold_list, xlabel=xlabel,
                                                     ylabel=ylabel, title_list=title_list, logbase=logbase,
                                                     label_list=label_list, extensions=extensions, suptitle=suptitle,
                                                     share_y_axis=share_y_axis, share_x_axis=share_x_axis)

    def draw_hexa_histogram_with_three_logscaled_from_file(self, list_of_files, column_idx_list, output_prefix,
                                                           figsize=(15, 10), number_of_bins_list=None,
                                                           width_of_bins_list=None,
                                                           max_threshold_list=None, min_threshold_list=None,
                                                           xlabel=None, ylabel=None,
                                                           title_list=None, logbase=10, label_list=None,
                                                           extensions=("png",), suptitle=None, separator=None,
                                                           share_y_axis=False, share_x_axis=False,
                                                           comments="#"):

        list_of_data_arrays = []
        for filename, column_idx in zip(list_of_files, column_idx_list):
            list_of_data_arrays.append(np.loadtxt(filename, usecols=(column_idx,),
                                                  delimiter=separator, comments=comments))

        self.draw_hexa_histogram_with_three_logscaled(list_of_data_arrays,  output_prefix=output_prefix,
                                                      figsize=figsize, number_of_bins_list=number_of_bins_list,
                                                      width_of_bins_list=width_of_bins_list,
                                                      max_threshold_list=max_threshold_list,
                                                      min_threshold_list=min_threshold_list, xlabel=xlabel,
                                                      ylabel=ylabel, title_list=title_list, logbase=logbase,
                                                      label_list=label_list, extensions=extensions, suptitle=suptitle,
                                                      share_y_axis=share_y_axis, share_x_axis=share_x_axis)

    @staticmethod
    def draw_histogram_from_file(input_file, output_prefix, number_of_bins=None, width_of_bins=None,
                                 separator="\n", max_length=None, min_length=1, xlabel=None, ylabel=None,
                                 title=None, extensions=("png",), logbase=10):
        if (number_of_bins is not None) and (width_of_bins is not None):
            raise AttributeError("Options -w/--width_of_bins and -b/--number_of_bins mustn't be set simultaneously")

        if min_length < 0:
            raise ValueError("Minimum length can't be negative")
        if (max_length is not None) and (max_length < 0):
            raise ValueError("Maximum length can't be negative")

        lengths = np.fromfile(input_file, sep=separator)

        max_lenn = max(lengths)
        min_lenn = min(lengths)

        #max_len = max_length if (max_length is not None) and (max_length < max_lenn) else max_lenn
        #min_len = min_length if (min_length is not None) and min_lenn < min_length else min_lenn

        max_len = max_length if (max_length is not None) else max_lenn
        min_len = min_length if (min_length is not None) else min_lenn

        #print(lengths)
        #print(max_len)
        #print(min_len)

        filtered = []

        if (max_len < max_lenn) and (min_len > min_lenn):
            for entry in lengths:
                if min_len <= entry <= max_len:
                    filtered.append(entry)
        elif max_len < max_lenn:
            for entry in lengths:
                if entry <= max_len:
                    filtered.append(entry)
        elif min_len > min_lenn:
            for entry in lengths:
                if min_len <= entry:
                    filtered.append(entry)
        else:
            filtered = lengths

        figure = plt.figure(1, figsize=(6, 6))
        subplot = plt.subplot(1, 1, 1)

        if number_of_bins:
            bins = number_of_bins
        elif width_of_bins:
            bins = np.arange(min_len, max_len, width_of_bins)
            #print bins
            #bins[0] += 1
            bins = np.append(bins, [max_len])
        else:
            bins = 30

        n, bins, patches = plt.hist(filtered, bins=bins)

        print("Minimum x in histogram: %f" % float(min_len))
        print("Maximum x in histogram: %f" % float(max_len))
        print("Bins:")
        print(bins)

        #bin_centers = (bins + ((bins[1] - bins[0])/2))[:-1]
        #print bin_centers
        #print len(n)
        #print len(bin_centers)

        plt.xlim(xmin=min_len, xmax=max_len)
        if xlabel:
            plt.xlabel(xlabel)
        if ylabel:
            plt.ylabel(ylabel)
        if title:
            plt.title(title)

        for ext in extensions:
            plt.savefig("%s.%s" % (output_prefix, ext))

        subplot.set_yscale('log', basey=logbase)
        #subplot.set_xscale('log', basex=args.logbase)
        for ext in extensions:
            plt.savefig("%s.logscale.%s" % (output_prefix, ext))

        # save histo values
        np.savetxt("%s.histo" % output_prefix, zip(bins[:-1], n), fmt="%i\t%i")
        np.savetxt("%s.bins" % output_prefix, bins)

    @staticmethod
    def generate_bin_array_by_width(min_value, max_value, bin_width, add_max_value=True):

        #print min_value, max_value, bin_width
        bin_array = np.arange(min_value, max_value, bin_width)
        if add_max_value:
            bin_array = np.append(bin_array, [max_value])

        return bin_array
    """
    def generate_bin_array(self, x, y, bin_number=20, bin_width=None, bin_array=None,
                           min_x_value=None, max_x_value=None, min_y_value=None, max_y_value=None, add_max_value=True):
        if (bin_width is not None) and (bin_array is not None):
            raise ValueError("Both bin width and bin array were set")

        min_x, max_x = min(x), max(x)
        min_y, max_y = min(y), max(y)

        if bin_width:
            xbins = self.generate_bin_array_by_width(min_x_value if min_x_value is not None else min_x,
                                                     max_x_value if max_x_value is not None else max_x,
                                                     bin_width if isinstance(bin_width, int) else bin_width[0],
                                                     add_max_value=add_max_value)
            ybins = self.generate_bin_array_by_width(min_y_value if min_y_value is not None else min_y,
                                                     max_y_value if max_y_value is not None else max_y,
                                                     bin_width if isinstance(bin_width, int) else bin_width[1],
                                                     add_max_value=add_max_value)
            bins = (xbins, ybins)

        elif bin_array:
            bins = bin_array
        else:
            xbins = np.linspace(min_x_value if min_x_value is not None else min_x,
                                max_x_value if max_x_value is not None else max_x,
                                bin_number if isinstance(bin_number, int) else bin_number[0])
            ybins = np.linspace(min_y_value if min_y_value is not None else min_y,
                                max_y_value if max_y_value is not None else max_y,
                                bin_number if isinstance(bin_number, int) else bin_number[1])
            bins = (xbins, ybins)

        return bins
    """

    def generate_bin_array(self, x_list, y_list=None, bin_number=20, bin_width=None, bin_array=None,
                           min_x_value=None, max_x_value=None, min_y_value=None,
                           max_y_value=None, add_max_value=True):
        if (bin_width is not None) and (bin_array is not None):
            raise ValueError("Both bin width and bin array were set")
        #print(x_list)
        min_x, max_x = min(map(min, x_list) if isinstance(x_list[0], Iterable) else x_list), \
                       max(map(max, x_list) if isinstance(x_list[0], Iterable) else x_list)
        if not(y_list is None):
            min_y, max_y = min(map(min, y_list) if isinstance(y_list[0], Iterable) else y_list), \
                           max(map(max, y_list) if isinstance(y_list[0], Iterable) else y_list)

        if bin_width:
            xbins = self.generate_bin_array_by_width(min_x_value if min_x_value is not None else min_x,
                                                     max_x_value if max_x_value is not None else max_x,
                                                     bin_width if isinstance(bin_width, int) else bin_width[0],
                                                     add_max_value=add_max_value)
            if not(y_list is None):
                ybins = self.generate_bin_array_by_width(min_y_value if min_y_value is not None else min_y,
                                                         max_y_value if max_y_value is not None else max_y,
                                                         bin_width if isinstance(bin_width, int) else bin_width[1],
                                                         add_max_value=add_max_value)
            bins = (xbins, ybins) if not(y_list is None) else xbins

        elif bin_array:
            bins = bin_array
        else:
            if bin_number is None:
                print("WARNINNG!!! No bin_number or bin_width or bin_array were set. "
                      "Instead default value(20) for bin number is used.")
            xbins = np.linspace(min_x_value if min_x_value is not None else min_x,
                                max_x_value if max_x_value is not None else max_x,
                                21 if bin_number is None else bin_number + 1 if isinstance(bin_number, int) else bin_number[0] + 1)
            if not(y_list is None):
                ybins = np.linspace(min_y_value if min_y_value is not None else min_y,
                                    max_y_value if max_y_value is not None else max_y,
                                    21 if bin_number is None else bin_number + 1 if isinstance(bin_number, int) else bin_number[1] + 1)
            bins = (xbins, ybins) if not(y_list is None) else xbins

        return bins

    def draw_heatmap(self, x, y, output_prefix=None, xlabel=None, ylabel=None, title=None,
                     figsize=(8, 8), minimum_counts_to_show=1,
                     extensions=("png", "svg"), show_colorbar=True, bin_number=20, bin_width=None, bin_array=None,
                     min_x_value=None, max_x_value=None, min_y_value=None, max_y_value=None, add_max_value=True,
                     subplot=None, save_histovalues_only=False, header=None, logscaled=False):
        """
        bin_width: numeric or tuple of two numerics
        """

        bins = self.generate_bin_array(x, y, bin_number=bin_number, bin_width=bin_width, bin_array=bin_array,
                                       min_x_value=min_x_value, max_x_value=max_x_value,
                                       min_y_value=min_y_value, max_y_value=max_y_value,
                                       add_max_value=add_max_value)
        print("X axis bins:")
        print(bins[0])
        print("Y axis bins:")
        print(bins[1])

        if subplot is None:
            #print "aaaaaaaaaa"
            #figure = plt.figure(1, figsize=figsize,)
            #subplott = figure.add_subplot(1, 1, 1)
            fig, ax = plt.subplots(figsize=figsize)
        else:
            ax = plt.axes(subplot)

        counts, xedges, yedges, image = plt.hist2d(x, y, bins, cmin=minimum_counts_to_show,
                                                   norm=LogNorm() if logscaled else None)
        print("X edges bins:")
        print(xedges)
        print("Y edges bins:")
        print(yedges)
        print(counts)
        #print x
        #print y
        #print minimum_counts_to_show
        plt.xlim(xmin=min_x_value, xmax=max_x_value)
        plt.ylim(ymin=min_y_value, ymax=max_y_value)

        if xlabel:
            plt.xlabel(xlabel)

        if ylabel:
            plt.ylabel(ylabel)

        if title:
            plt.title(title)

        #np.savetxt("tara.t", counts, delimiter='\n')
        if show_colorbar:
            max_counts = np.nanmax(counts)
            #print max_counts
            plt.colorbar(image, ax=ax)
            """
            cmap = plt.get_cmap('jet', max_counts)
            #cmap.set_under('gray')
            mappable = plt.cm.ScalarMappable(cmap=cmap)
            mappable.set_array([])
            mappable.set_clim(1, max_counts + 1)
            #mappable.set_array([])
            #mappable.set_clim(-0.5, ncolors+0.5)
            colorbar = plt.colorbar(mappable)
            #colorbar.set_ticks(np.linspace(1 + 0.5, max_counts + 0.5, max_counts), minor=True)
            decimal = int(max_counts / 10) + 1
            max_major_tick = max_counts - (max_counts % decimal)
            major_ticks = np.linspace(decimal + 0.5, max_major_tick + 0.5, int(max_major_tick / decimal))
            colorbar.set_ticks(major_ticks)
            print (decimal, max_major_tick + decimal, decimal)
            colorbar.set_ticklabels(range(decimal, int(max_major_tick + decimal), decimal))
            """
        if output_prefix:
            if not save_histovalues_only:
                for ext in extensions:
                    plt.savefig("%s.%s" % (output_prefix, ext))
            with open("%s.2d_histo" % output_prefix, "w") as out_fd:
                if header:
                    out_fd.write(header if header[-1] == "\n" else header + "\n")
                for ix in range(0, len(xedges) - 1):
                    for iy in range(0, len(yedges) - 1):
                        out_fd.write("%d\t%d\t%d\n" % (xedges[ix], yedges[iy], 0 if np.isnan(counts[ix, iy]) else counts[ix, iy]))

        return counts, xedges, yedges

    def draw_percent_heatmap(self, x, y, output_prefix=None, xlabel=None, ylabel=None, title=None,
                             figsize=(8, 8), minimum_counts_to_show=1,
                             extensions=("png", "svg"), show_colorbar=True, bin_number=20, bin_width=None, bin_array=None,
                             type="percent", add_max_value=True,
                             subplot=None, save_histovalues_only=False, header=None, logscaled=False):
        if type == "percent":
            return self.draw_heatmap(x, y, output_prefix, xlabel=xlabel, ylabel=ylabel, title=title,
                                     figsize=figsize, minimum_counts_to_show=minimum_counts_to_show,
                                     extensions=extensions, show_colorbar=show_colorbar, bin_number=bin_number,
                                     bin_width=bin_width, bin_array=bin_array, min_x_value=0,
                                     max_x_value=100, min_y_value=0, max_y_value=100,
                                     add_max_value=add_max_value, subplot=subplot,
                                     save_histovalues_only=save_histovalues_only, header=header,
                                     logscaled=logscaled)
        elif type == "fraction":
            return self.draw_heatmap(x, y, output_prefix, xlabel=xlabel, ylabel=ylabel, title=title,
                                     figsize=figsize, minimum_counts_to_show=minimum_counts_to_show,
                                     extensions=extensions, show_colorbar=show_colorbar, bin_number=bin_number,
                                     bin_width=bin_width, bin_array=bin_array, min_x_value=0.0,
                                     max_x_value=1.0, min_y_value=0.0, max_y_value=1.0,
                                     add_max_value=add_max_value, subplot=subplot,
                                     save_histovalues_only=save_histovalues_only, header=header,
                                     logscaled=logscaled)
        else:
            raise ValueError("Unrecognized type of percent histogram(neither fraction nor percent)")

    def draw_heatmap_from_file(self, input_file, output_prefix, x_column=0, y_column=1, xlabel=None, ylabel=None,
                               title=None, figsize=(8, 8), minimum_counts_to_show=1, extensions=("png", "svg"),
                               show_colorbar=True, bin_number=20, bin_width=None, bin_array=None,
                               min_x_value=None, max_x_value=None, min_y_value=None, max_y_value=None,
                               add_max_value=True):

        x = np.loadtxt(input_file, usecols=(x_column,))
        y = np.loadtxt(input_file, usecols=(y_column,))

        self.draw_heatmap(x, y, output_prefix=output_prefix, xlabel=xlabel, ylabel=ylabel, title=title,
                          figsize=figsize, minimum_counts_to_show=minimum_counts_to_show, extensions=extensions,
                          show_colorbar=show_colorbar, bin_number=bin_number, bin_width=bin_width, bin_array=bin_array,
                          min_x_value=min_x_value, max_x_value=max_x_value,
                          min_y_value=min_y_value, max_y_value=max_y_value,
                          add_max_value=add_max_value,)

    def draw_double_histo_from_file(self, file_list, column_idx_list, subplot_tuple=(2, 1), output_prefix=None,
                                    figsize=(5, 10), number_of_bins_list=None, width_of_bins_list=None,
                                    max_threshold_list=None, min_threshold_list=None, xlabel_list=None, ylabel_list=None,
                                    title_list=None, ylogbase_list=None, label_list=None,
                                    extensions=("png",), suptitle=None, separator=None, comments='#',
                                    share_y_axis=False,
                                    share_x_axis=False):
        list_of_data_arrays = []
        for filename, column_idx in zip(file_list, column_idx_list):
            #print filename, column_idx
            list_of_data_arrays.append(np.loadtxt(filename, usecols=(column_idx,),
                                                  delimiter=separator, comments=comments))

        self.draw_multi_histogram_picture(list_of_data_arrays, subplot_tuple, output_prefix=output_prefix,
                                          figsize=figsize, number_of_bins_list=number_of_bins_list,
                                          width_of_bins_list=width_of_bins_list,
                                          max_threshold_list=max_threshold_list, min_threshold_list=min_threshold_list,
                                          xlabel_list=xlabel_list, ylabel_list=ylabel_list,
                                          title_list=title_list, ylogbase_list=ylogbase_list, label_list=label_list,
                                          extensions=extensions, suptitle=suptitle,
                                          share_y_axis=share_y_axis, share_x_axis=share_x_axis)

    @staticmethod
    def venn_diagram_from_sets(set1, set2, set3=None, set_labels=None, set_colors=None,
                               output_prefix=None, extensions=("png",), title=None):
        if set3 is None:
            diagramm = venn2(subsets=(set1, set2), set_labels=set_labels, set_colors=set_colors)
        else:
            diagramm = venn3(subsets=(set1, set2, set3), set_labels=set_labels, set_colors=set_colors)

        if title:
            plt.title(title)

        if output_prefix:
            for ext in extensions:
                plt.savefig("%s.%s" % (output_prefix, ext))

        return diagramm

    def venn_diagram_from_sets_from_files(self, set1_file, set2_file, set3_file=None, set_labels=None, set_colors=None,
                                          output_prefix=None, extensions=("png",), title=None):
        set1 = IdSet(filename=set1_file)
        set2 = IdSet(filename=set2_file)
        set3 = IdSet(filename=set3_file) if set3_file else None

        return self.venn_diagram_from_sets(set1, set2, set3=set3, set_labels=set_labels, set_colors=set_colors,
                                           output_prefix=output_prefix, extensions=extensions, title=title)

    def draw_bar_plot(self, input_data, output_prefix, extentions=["png"],
                      xlabel=None, ylabel=None, title=None, min_value=None, max_value=None, new_figure=True,
                      figsize=(6, 6), close_figure=True):
        
        data = np.array(input_data)
        if min_value and max_value:
            data = data[(data <= min_value) & (data >= max_value)]
        elif max_value:
            data = data[data <= max_value]
        elif min_value:
            data = data[data >= min_value]
        if new_figure:
            plt.figure(1, figsize=figsize)
            plt.subplot(1, 1, 1)
        plt.bar(np.arange(1, len(data) + 1, 1), data)
        plt.xlim(xmin=0, xmax=len(data))
        
        if xlabel:
            plt.xlabel(xlabel)
        if ylabel:
            plt.ylabel(ylabel)
        if title:
            plt.title(title)
        
        for ext in extentions:
            plt.savefig("%s.%s" % (output_prefix, ext))

        plt.close()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from collections import OrderedDict
from RouToolPa.Collections.General import TwoLvlDict, IdSet

import matplotlib.pyplot as plt

from RouToolPa.Parsers.BUSCO import BUSCOtable
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import MatplotlibRoutines



class BUSCO(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "run_BUSCO.py", path=path, max_threads=max_threads)
        self.kingdoms = ["euk", "bac", "mito", "arc"]
        self.status_list = ["Complete",
                            "Duplicated",
                            "Fragmented",
                            "Missing"]

    def parse_options(self, input_fasta, label, busco_db, augustus_species, mode=None):

        options = " -c %i" % self.threads
        options += " -l %s" % busco_db
        options += " -m %s" % mode if mode else ""
        options += " -o %s" % label if label else ""
        options += " -sp %s" % augustus_species if augustus_species else ""
        options += " -i %s" % input_fasta

        return options

    def assess(self, input_fasta, label, busco_db, augustus_species, mode=None):

        options = self.parse_options(input_fasta, label, busco_db, augustus_species, mode=mode)

        self.execute(options=options)

    def assess_multiple_genomes(self, input_fasta_list, busco_db, augustus_species, label_list=None,
                                output_dir="./", mode=None):

        outdir = os.path.abspath(output_dir)

        for fasta, label in zip(input_fasta_list, label_list if label_list else ["A%i" % i for i in range(1,
                                                                                                          len(input_fasta_list) + 1)]):
            genome_fasta = os.path.abspath(fasta)
            options = self.parse_options(genome_fasta, label, busco_db, augustus_species, mode=mode)
            busco_dir = "%s/%s/" % (outdir, label)
            self.safe_mkdir(busco_dir)

            self.execute(options=options, directory=busco_dir)

    def compare_multiple_genome_results(self, busco_file_list, output_prefix, label_list=None,
                                        black_scaffold_list=(), white_scaffold_list=()):

        busco_table_dict = OrderedDict()
        gene_id_dict = OrderedDict()
        counts_dict = OrderedDict()

        output_path_list = self.split_filename(output_prefix)

        pairwise_overlaps_dir = "%s/pairwise_overlaps/" % (output_path_list[0] if output_path_list[0] else ".")
        pairwise_overlap_counts_dir = "%s/pairwise_overlap_counts/" % (output_path_list[0] if output_path_list[0] else ".")
        self.safe_mkdir(pairwise_overlaps_dir)
        self.safe_mkdir(pairwise_overlap_counts_dir)

        lllabels_list = label_list if label_list else ["A%i" % i for i in range(1, len(busco_file_list) + 1)]

        for busco_table, label in zip(busco_file_list, lllabels_list):
            busco_table_dict[label] = BUSCOtable(in_file=busco_table, black_list=black_scaffold_list,
                                                 white_list=white_scaffold_list)

            gene_id_dict[label] = OrderedDict()
            counts_dict[label] = OrderedDict()

            gene_id_dict[label], counts_dict[label] = busco_table_dict[label].count_statuses()

        # TODO: draw piecharts


        # TODO: count overlaps

        pairwise_overlap_dict = OrderedDict()
        count_pairwise_overlap_dict = OrderedDict()
        for label1 in lllabels_list:
            for label2 in lllabels_list:
                if label1 == label2:
                    continue
                overlap_id = "%s_vs_%s" % (label1, label2)
                pairwise_overlap_dict[overlap_id] = TwoLvlDict()
                count_pairwise_overlap_dict[overlap_id] = TwoLvlDict()
                for status1 in self.status_list:
                    pairwise_overlap_dict[overlap_id]["%s@%s" % (label1, status1)] = OrderedDict()
                    count_pairwise_overlap_dict[overlap_id]["%s@%s" % (label1, status1)] = OrderedDict()
                    for status2 in self.status_list:
                        pairwise_overlap_dict[overlap_id]["%s@%s" % (label1, status1)]["%s@%s" % (label2, status2)] = IdSet(gene_id_dict[label1][status1] & gene_id_dict[label2][status2])
                        count_pairwise_overlap_dict[overlap_id]["%s@%s" % (label1, status1)]["%s@%s" % (label2, status2)] = len(pairwise_overlap_dict[overlap_id]["%s@%s" % (label1, status1)]["%s@%s" % (label2, status2)])
                        pairwise_overlap_dict[overlap_id]["%s@%s" % (label1, status1)]["%s@%s" % (label2, status2)].write("%s/%s.%s_vs_%s.ids" % (pairwise_overlaps_dir, output_prefix, "%s@%s" % (label1, status1), "%s@%s" % (label2, status2)))

                count_pairwise_overlap_dict[overlap_id].write("%s/%s.overlap.%s.tsv" % (pairwise_overlap_counts_dir, output_prefix, overlap_id))

        if 2 <= len(busco_file_list) <= 3:
            fig, subplot_list = plt.subplots(2, 2, figsize=(6, 6))
            plt.suptitle("Overlaps for BUSCO categories between assemblies/genomes")
            print subplot_list
            for status, index in zip(self.status_list, range(0, 4)):

                plt.sca(subplot_list[index // 2][index % 2])
                plt.title(status)
                MatplotlibRoutines.venn_diagram_from_sets(gene_id_dict[lllabels_list[0]][status],
                                                          gene_id_dict[lllabels_list[1]][status],
                                                          set3=gene_id_dict[lllabels_list[2]][status] if len(lllabels_list) > 2 else None,
                                                          set_labels=lllabels_list, set_colors=["red", "yellow", "green"],
                                                          output_prefix=None, extensions=("png",), title=None)

            plt.savefig("%s.venn.png" % output_prefix)

            plt.close()

if __name__ == "__main__":
    pass



__author__ = 'mahajrod'
import os
import sys
from collections import OrderedDict

if sys.version_info[0] == 3:
    izip = zip
else:
    from itertools import izip

import pandas as pd
from Bio import SearchIO, SeqIO, AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Align import MultipleSeqAlignment
from RouToolPa.Routines.Sequence import SequenceRoutines
#from Data.Nucleotides import back_degenerate_nucleotides
from RouToolPa.Collections.General import SynDict, IdList

import numpy as np

class AlignmentRoutines(SequenceRoutines):
    def __init__(self):
        self.psl_query_id_column = 9           # zero-based
        self.psl_target_id_column = 13         # zero-based
        pass

    """
    @staticmethod
    def get_db_ids(search_dict):
        id_set = set()
        for query_id in search_dict:
            for hit in search_dict[query_id]:
                id_set.add(hit.id)
        return id_set

    @staticmethod
    def get_codon_alignment(protein_alignment, nucleotide_seq_dict, codon_alignment_file,
                            protein2cds_accordance_dict=None):
        codon_alignment = {}
        if protein2cds_accordance_dict:
            for record in protein_alignment:
                if record.id not in protein2cds_accordance_dict:
                    print("Corresponding CDS was not found for %s protein" % record.id)
                    return -1
        for record in protein_alignment:
            nucleotide_seq = ""
            i = 0
            for aminoacid in record.seq:
                if aminoacid == "-":
                    nucleotide_seq += "---"
                    continue
                else:
                    nucleotide_seq += str(nucleotide_seq_dict[protein2cds_accordance_dict[record.id] if protein2cds_accordance_dict else record.id].seq[3*i:3*(i+1)])
                    i += 1
            codon_alignment[record.id] = SeqRecord(Seq(nucleotide_seq),
                                                   id=record.id,
                                                   description=record.description,
                                                   name=record.name)
            #print(record.id, record.seq)
        SeqIO.write(list(codon_alignment.values()), codon_alignment_file, "fasta")
        return codon_alignment

    def get_codon_alignment_from_files(self, protein_aln_file, nucleotide_seq_file, codon_alignment_file,
                                       cds2protein_accordance_file=None,
                                       alignment_format="fasta", nucleotide_sequence_format="fasta",
                                       cds_index_file=None, retain_cds_index=False):
        protein_aln_dict = AlignIO.read(protein_aln_file, format=alignment_format)
        nucleotide_seq_dict = SeqIO.index_db(cds_index_file if cds_index_file else "nuc_tmp.idx", nucleotide_seq_file,
                                             format=nucleotide_sequence_format)

        protein2cds_accordance_dict = None
        if cds2protein_accordance_file:
            protein2cds_accordance_dict = SynDict()
            protein2cds_accordance_dict.read(cds2protein_accordance_file, key_index=1, value_index=0)

        self.get_codon_alignment(protein_aln_dict, nucleotide_seq_dict, codon_alignment_file,
                                 protein2cds_accordance_dict=protein2cds_accordance_dict)
        if (not cds_index_file) and (not retain_cds_index):
            os.remove("nuc_tmp.idx")

    @staticmethod
    def merge_alignment(alignment_file_list, merged_alignment_file, coordinates_file, format="fasta"):
        #print("Merging alignments...")
        alignment_list = []
        sequence_lengthes = []
        #print(alignment_file_list)

        alignment_file_list_sorted = sorted(alignment_file_list)
        #print(alignment_file_list_sorted)
        for alignment_file in alignment_file_list_sorted:
            #alignment_file.sort()
            parsed = AlignIO.read(alignment_file, format=format)
            parsed.sort()
            alignment_list.append(parsed)
        merged_alignment = None
        for alignment in alignment_list:
            if not merged_alignment:
                sequence_lengthes.append(alignment.get_alignment_length())
                merged_alignment = alignment
                continue
            #print(alignment)
            sequence_lengthes.append(alignment.get_alignment_length())
            merged_alignment += alignment
        SeqIO.write(merged_alignment, merged_alignment_file, "fasta")
        sequence_coordinates = []
        #
        for seq_length in sequence_lengthes:
            if not sequence_coordinates:
                sequence_coordinates.append((1, seq_length))
                continue
            sequence_coordinates.append((sequence_coordinates[-1][1]+1, sequence_coordinates[-1][1]+seq_length))
        #print(sequence_coordinates)
        with open(coordinates_file, "w") as coord_fd:
            coord_fd.write("#length\tstart\tend\n")
            for coord_tuple in sequence_coordinates:
                coord_fd.write("%i\t%i\t%i\n" % (coord_tuple[1] - coord_tuple[0] + 1, coord_tuple[0], coord_tuple[1]))
        return merged_alignment, sequence_lengthes, sequence_coordinates

    @staticmethod
    def extract_degenerate_sites_from_codon_alignment(alignment, genetic_code_table=1):
        degenerate_codon_set = SequenceRoutines.get_degenerate_codon_set(genetic_code_table)
        number_of_alignments = len(alignment)
        alignment_length = len(alignment[0])
        if alignment_length % 3 > 0:
            raise(ValueError, "Length of alignment is not divisible by 3")
        else:
            number_of_codons = int(alignment_length / 3)
        degenerate_columns = []
        for i in range(0, number_of_codons):
            position_strings = []
            for j in range(0, 3):
                position_strings.append(list(set(alignment[:, 3*i + j])))
            if (len(position_strings[0]) > 1) or (len(position_strings[1]) > 1):
                continue
            ambigious_codon = position_strings[0][0] + position_strings[1][0] + "N"

            #if Seq(ambigious_codon).translate(table=genetic_code_table) == "X":
            #    continue
            #else:
            #    degenerate_columns.append(alignment[:, 3*i + 2])
            #    #print(i*3 +3)

            if ambigious_codon in degenerate_codon_set:
                degenerate_columns.append(alignment[:, 3*i + 2])

        number_of_degenerate_columns = len(degenerate_columns)
        record_list = []
        for i in range(0, number_of_alignments):
            string = ""
            for j in range(0, number_of_degenerate_columns):
                string += degenerate_columns[j][i]
            record = SeqRecord(seq=Seq(string), id=alignment[i].id)
            record_list.append(record)

        #print(number_of_alignments, alignment_length)
        degenerate_alignment = MultipleSeqAlignment(record_list)

        return degenerate_alignment

    def extract_degenerate_sites_from_codon_alignment_from_file(self, alignment_file, output_alignment_file,
                                                                genetic_code_table=1, format="fasta"):
        alignment = AlignIO.read(alignment_file, format=format)
        degenerate_alignment = self.extract_degenerate_sites_from_codon_alignment(alignment,
                                                                                  genetic_code_table=genetic_code_table)

        AlignIO.write([degenerate_alignment], output_alignment_file, format=format)

    @staticmethod
    def sequences_from_alignment_generator(alignments, gap_symbol="-"):
        for record in alignments:
            record.seq = Seq(str(record.seq).replace(gap_symbol, ""))
            yield record

    def extract_sequences_from_alignment(self, alignment_file, output_file, alignment_format="fasta",
                                         output_format="fasta", gap_symbol="-"):
        alignments = AlignIO.read(alignment_file, format=alignment_format)
        SeqIO.write(self.sequences_from_alignment_generator(alignments, gap_symbol=gap_symbol),
                    output_file, format=output_format)
    """

    def filter_psl_by_ids(self, psl_file, output_file,
                          white_query_id_list=(),
                          black_query_id_list=(),
                          white_target_id_list=(),
                          black_target_id_list=()):

        with self.metaopen(psl_file, "r") as in_fd, \
             self.metaopen(output_file, "w") as out_fd:

            for i in range(0, 20):
                line = in_fd.readline()
                if line[:3] == "---":
                    break
            else:
                in_fd.seek(0)

            for line in in_fd:
                tmp_line = line.split("\t")
                if self.check_id(tmp_line[self.psl_query_id_column],
                                 white_query_id_list,
                                 black_query_id_list) and \
                   self.check_id(tmp_line[self.psl_target_id_column],
                                 white_target_id_list,
                                 black_target_id_list):

                    out_fd.write(line)

    def filter_psl_by_ids_from_file(self, psl_file, output_file,
                                    white_query_id_file=None,
                                    black_query_id_file=None,
                                    white_target_id_file=None,
                                    black_target_id_file=None):

        self.filter_psl_by_ids(psl_file, output_file,
                               white_query_id_list=IdList(filename=white_query_id_file) if white_query_id_file else (),
                               black_query_id_list=IdList(filename=black_query_id_file) if black_query_id_file else (),
                               white_target_id_list=IdList(filename=white_target_id_file) if white_target_id_file else (),
                               black_target_id_list=IdList(filename=black_target_id_file) if black_target_id_file else ())

    def filter_psl_by_target_len(self, psl_file, output_file, min_target_len=None, max_target_len=None):
        pass

    @staticmethod
    def parse_search_file(input_file, mode, format="hmmer3-text", index_file=None):
        if mode == "index_db" or ((not isinstance(input_file, str)) and (len(input_file) > 1)):
            index = index_file if index_file else "tmp.idx"
            seq_dict = SearchIO.index_db(index, [input_file] if isinstance(input_file, str) else input_file, format=format)
        elif mode == "index":
            seq_dict = SearchIO.index(input_file if isinstance(input_file, str) else input_file[0], format=format)
        elif mode == "parse":
            seq_dict = OrderedDict()
            for record in SearchIO.parse(input_file if isinstance(input_file, str) else input_file[0], format=format):
                seq_dict[record.id] = record
            #seq_dict = SeqIO.to_dict(SeqIO.parse(input_file if isinstance(input_file, str) else input_file[0], format=format))

        return seq_dict

    def calculate_masking_from_coverage_files(self, coverage_file_list, mean_coverage_list, output_file,
                                              sample_labels=None,
                                              max_threshold=2.5, min_threshold=None,min_sample_number=1,
                                              scaffold_column=0, position_column=1, coverage_column=2):
        """

        :param coverage_file_list:
        :param mean_coverage_list:
        :param output_file:
        :param sample_labels:
        :param max_threshold:
        :param min_threshold:
        :param min_sample_number:
        :param scaffold_column: 0-based
        :param position_column: 0-based
        :param coverage_column: 0-based
        :return:
        """
        file_number = len(coverage_file_list)
        index_list = [i for i in range(0, file_number)]
        if min_sample_number > file_number:
            raise ValueError("ERROR!!! Minimun number of files necessary for treating position as masked is greater "
                             "than total number of files")

        if max_threshold and min_threshold:
            max_coverage_list = np.array(mean_coverage_list) * max_threshold
            min_coverage_list = np.array(mean_coverage_list) * min_threshold

            def check_pos(entry):
                if np.sum(np.logical_and(np.less(entry, min_coverage_list),
                                         np.greater(entry, max_coverage_list))) >= min_sample_number:
                    return True
                return False

        elif max_threshold:
            max_coverage_list = np.array(mean_coverage_list) * max_threshold

            def check_pos(entry):
                if np.sum(np.greater(entry, max_coverage_list)) >= min_sample_number:
                    return True
                return False

        elif min_threshold:
            min_coverage_list = np.array(mean_coverage_list) * min_threshold

            def check_pos(entry):
                if np.sum(np.less(entry, min_coverage_list)) >= min_sample_number:
                    return True
                return False
        else:
            raise ValueError("ERROR!!! Neither minimum nor maximum threshold was set!")

        line_index = 1
        with open(output_file, "w") as out_fd:
            out_fd.write("#scaffold\tposition\t%s\n" % (",".join(sample_labels if sample_labels else coverage_file_list)))

            for line_list in izip(*[self.file_line_as_list_generator(filename) for filename in coverage_file_list]):

                coverage_list = [int(line_list[i][coverage_column]) for i in index_list]
                if check_pos(coverage_list):
                    out_fd.write("%s\t%s\t%s\n" % (line_list[0][scaffold_column],
                                                   line_list[0][position_column],
                                                   ",".join(map(str, coverage_list))))

                if line_index % 100000000 == 0:
                    print("Handled %i million positions..." % (line_index/1000000))

                line_index += 1

    def collapse_per_base_coverage_mask(self, mask_file, out_file,
                                        scaffold_column=0,
                                        position_column=1,
                                        comments_prefix="#",
                                        zero_based_output=False,
                                        in_memory=True):

        line_list_generator = self.file_line_as_list_generator(mask_file, comments_prefix=comments_prefix)
        with self.metaopen(out_file, "w") as out_fd:
            tmp = next(line_list_generator)

            prev_scaffold = tmp[scaffold_column]
            prev_start = int(tmp[position_column])
            prev_end = int(tmp[position_column])

            coordinates_df = []
            if in_memory:
                for line_list in line_list_generator:
                    pos = int(line_list[position_column])
                    if (prev_scaffold != line_list[scaffold_column]) or (pos != prev_end + 1):
                        coordinates_df.append((prev_scaffold, prev_start, prev_end))

                        prev_scaffold = line_list[scaffold_column]
                        prev_start = pos
                        prev_end = pos
                    else:
                        prev_end += 1

                coordinates_df.append((prev_scaffold, prev_start, prev_end))
                coordinates_df = pd.DataFrame(coordinates_df, columns=("scaffold", "start", "end"))
                coordinates_df.set_index("scaffold", inplace=True)
                if zero_based_output:
                    coordinates_df["start"] -= 1

                coordinates_df.to_csv(out_fd, sep="\t", index=True)
            else:
                for line_list in line_list_generator:
                    pos = int(line_list[position_column])
                    if (prev_scaffold != line_list[scaffold_column]) or (pos != prev_end + 1):
                        out_fd.write("%s\t%i\t%i\n" % (prev_scaffold, (prev_start - 1) if zero_based_output == "0-based" else prev_start, prev_end))

                        prev_scaffold = line_list[scaffold_column]
                        prev_start = pos
                        prev_end = pos
                    else:
                        prev_end += 1

                out_fd.write("%s\t%i\t%i\n" % (prev_scaffold, (prev_start - 1) if zero_based_output == "0-based" else prev_start, prev_end))



#!/usr/bin/env python
import os
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import AnnotationsRoutines
from RouToolPa.Collections.General import SynDict


class FastaAlternateReferenceMaker4(Tool):
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_fasta_FastaAlternateReferenceMaker.html

    def __init__(self, max_threads=4,  max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk --java-options -Xmx%s FastaAlternateReferenceMaker" % max_memory if max_memory else "gatk FastaAlternateReferenceMaker",
                      max_threads=max_threads, max_memory=max_memory,
                      timelog=timelog)

    @staticmethod
    def parse_options(reference, new_reference, variants_vcf, raw_seq_per_line=False, vcf_with_masking=None,
                      override_vcf_by_mask=None, use_ambiguous_nuccleotides=None, interval_list=None):
        options = " -R %s" % reference
        options += " -O %s" % new_reference
        options += " --variant %s" % variants_vcf
        options += " --rawOnelineSeq" if raw_seq_per_line else ""
        options += " --snpmask %s" % vcf_with_masking if vcf_with_masking else ""
        options += " --snpmaskPriority" if override_vcf_by_mask else ""
        options += " --use_IUPAC" if use_ambiguous_nuccleotides else ""

        if interval_list:
            if isinstance(interval_list, str):
                options += " -L %s" % interval_list
            else:
                for entry in interval_list:
                    options += " -L %s" % entry

        return options

    def correct_reference(self,
                          reference,
                          new_reference,
                          variants_vcf,
                          raw_seq_per_line=False,
                          vcf_with_masking=None,
                          override_vcf_by_mask=None,
                          use_ambiguous_nuccleotides=None,
                          interval_list=None):

        options = self.parse_options(reference,
                                     new_reference,
                                     variants_vcf,
                                     raw_seq_per_line=raw_seq_per_line,
                                     vcf_with_masking=vcf_with_masking,
                                     override_vcf_by_mask=override_vcf_by_mask,
                                     use_ambiguous_nuccleotides=use_ambiguous_nuccleotides,
                                     interval_list=interval_list)

        self.execute(options=options)

        #os.system("java -Xmx2g -jar %sGenomeAnalysisTK.jar -R %s -T FastaAlternateReferenceMaker -o %s --variant %s"
        #          % (gatk_dir, reference, new_reference, variants_vcf))

    def correct_regions_from_gff(self,
                                 reference,
                                 variants_vcf,
                                 gff_file,
                                 output_prefix=None,
                                 feature_type_list=["CDS"],
                                 unification_key="Parent",
                                 #raw_seq_per_line=False,
                                 vcf_with_masking=None,
                                 override_vcf_by_mask=None,
                                 use_ambiguous_nuccleotides=None):

        feature_dict = AnnotationsRoutines.get_feature_dict(gff_file,
                                                            output_prefix=output_prefix,
                                                            feature_type_list=feature_type_list,
                                                            unification_key=unification_key)
        region_file = "%s.coordinates_only.list" % output_prefix

        raw_regions = "%s.raw.seq" % output_prefix
        final_regions = "%s.fasta" % output_prefix

        regions_with_frameshift_file = "%s.frameshifts.region.ids" % output_prefix

        self.correct_reference(reference,
                               raw_regions,
                               variants_vcf,
                               raw_seq_per_line=True,
                               vcf_with_masking=vcf_with_masking,
                               override_vcf_by_mask=override_vcf_by_mask,
                               use_ambiguous_nuccleotides=use_ambiguous_nuccleotides,
                               interval_list=region_file)

        region_with_frameshift = SynDict()

        def new_regions_generator():
            with open(raw_regions, "r") as in_fd:
                for region_id in feature_dict:
                    seq = ""
                    for i in range(0, len(feature_dict[region_id])):
                        seq_fragment = in_fd.readline().strip()
                        if ((int(feature_dict[region_id][i][2]) - int(feature_dict[region_id][i][1]) + 1) - len(seq_fragment)) % 3 != 0:
                            if region_id not in region_with_frameshift:
                                region_with_frameshift[region_id] = [i]
                            else:
                                region_with_frameshift[region_id].append(i)
                        seq += seq_fragment
                    yield SeqRecord(seq=Seq(seq) if feature_dict[region_id][0][3] == "+" else Seq(seq).reverse_complement(),
                                    id=region_id,
                                    description="")

        SeqIO.write(new_regions_generator(), final_regions, format="fasta")
        region_with_frameshift.write(regions_with_frameshift_file, splited_values=True)

    def restore_names(self, reference, new_reference, corrected_reference, sed_script="sed_script.scr", names_file="chr_name_lines.t"):
        os.system("grep '>' %s > %s" % (reference, names_file))
        with open(names_file, "r") as in_fd:
            with open(sed_script, "w") as out_fd:
                line_number = 1
                for line in in_fd:
                    out_fd.write("s/>%i$/%s/\n" % (line_number, line.strip()))
                    line_number += 1
        os.system("sed -f %s %s > %s" % (sed_script, new_reference, corrected_reference))

    def convert_picard_dict_to_syn_file(self, picard_dict, output):
        with self.metaopen(picard_dict, "r") as in_fd, self.metaopen(output, "w") as out_fd:
            index = 1
            for line in in_fd:
                if line[:3] != "@SQ":
                    continue
                seq_id = line.split()[1][3:]
                out_fd.write("%s\t%i\n" % (seq_id, index))
                index += 1



#!/usr/bin/env python
__author__ = 'mahajrod'
from collections import OrderedDict
from RouToolPa.Collections.General import TwoLvlDict


class KrATERReport(OrderedDict):
    def __init__(self, krater_report_file):
        OrderedDict.__init__(self)
        #self = OrderedDict()

        with open(krater_report_file, "r") as in_fd:
            for line in in_fd:
                tmp = line.strip().split("\t")
                self[tmp[0]] = tmp[1]

        self["Number of distinct kmers"] = int(self["Number of distinct kmers"])
        self["Number of distinct kmers with errors"] = int(self["Number of distinct kmers with errors"])
        self["Fraction of distinct kmers with errors"] = float(self["Fraction of distinct kmers with errors"])

        self["Total number of kmers"] = int(self["Total number of kmers"])
        self["Total number of kmers with errors"] = int(self["Total number of kmers with errors"])
        self["Fraction of kmers with errors"] = float(self["Fraction of kmers with errors"])

        self["Width of first peak"] = int(self["Width of first peak"])
        self["Mean kmer multiplicity in first peak"] = float(self["Mean kmer multiplicity in first peak"])
        self["Kmer multiplicity at first maximum"] = float(self["Kmer multiplicity at first maximum"])
        self["Standard deviation of kmer multiplicity in first peak"] = float(self["Standard deviation of kmer multiplicity in first peak"])
        self["Variance coefficient of kmer multiplicity in first peak"] = float(self["Variance coefficient of kmer multiplicity in first peak"])
        if "Estimated genome size, bp" in self: # for compatibility with older versions that don't count genome size
            self["Estimated genome size, bp"] = float(self["Estimated genome size, bp"]) if self["Estimated genome size, bp"] != 'NA' else None


class KrATERReportCollection(OrderedDict):
    def __init__(self, report_dict=None, file_dict=None):
        OrderedDict.__init__(self)
        if report_dict:
            for report_id in report_dict:
                self[report_id] = report_dict
        if file_dict:
            for file_id in file_dict:
                if file_id in self:
                    raise KeyError("Report with same id(%s) is already present in KrATERReportCollection" % str(file_id))
                self[file_id] = KrATERReport(file_dict[file_id])

    def get_general_stats(self):
        stat_dict = TwoLvlDict()

        for report_id in self:
            stat_dict[report_id] = OrderedDict()

            stat_dict[report_id]["Number of distinct kmers"] = self[report_id]["Number of distinct kmers"]
            stat_dict[report_id]["Number of distinct kmers"] = self[report_id]["Number of distinct kmers"]
            stat_dict[report_id]["Fraction of distinct kmers with errors"] = self[report_id]["Fraction of distinct kmers with errors"]

            stat_dict[report_id]["Total number of kmers"] = self[report_id]["Total number of kmers"]
            stat_dict[report_id]["Total number of kmers with errors"] = self[report_id]["Total number of kmers with errors"]
            stat_dict[report_id]["Fraction of kmers with errors"] = self[report_id]["Fraction of kmers with errors"]

            stat_dict[report_id]["Width of first peak"] = self[report_id]["Width of first peak"]
            stat_dict[report_id]["Mean kmer multiplicity in first peak"] = self[report_id]["Mean kmer multiplicity in first peak"]
            stat_dict[report_id]["Kmer multiplicity at first maximum "] = self[report_id]["Kmer multiplicity at first maximum"]
            stat_dict[report_id]["Standard deviation of kmer multiplicity in first peak"] = self[report_id]["Standard deviation of kmer multiplicity in first peak"]
            stat_dict[report_id]["Variance coefficient of kmer multiplicity in first peak"] = self[report_id]["Variance coefficient of kmer multiplicity in first peak"]
            if "Estimated genome size, bp" in self[report_id]:
                stat_dict[report_id]["Estimated genome size,bp"] = self[report_id]["Estimated genome size, bp"]

        return stat_dict

    def write_general_stats(self, output_file):
        stat_dict = self.get_general_stats()
        stat_dict.write(output_file, absent_symbol=".", sort=False)





from RouToolPa.Tools.LinuxTools.CGAS import CGAS
from RouToolPa.Tools.LinuxTools.Download import *
from RouToolPa.Tools.LinuxTools.Valgrind import Valgrind


Axel = Axel()
Wget = Wget()
CGAS = CGAS()
Valgrind = Valgrind()



#!/usr/bin/env python
import os
from Bio import SeqIO
from RouToolPa.Collections.General import IdList, SynDict
from RouToolPa.Routines.SequenceCluster import SequenceClusterRoutines


class TreeFamRoutines(SequenceClusterRoutines):
    def __init__(self):

        pass

    @staticmethod
    def extract_proteins_from_selected_families(families_id_file, fam_file, pep_file,
                                                output_dir="./", pep_format="fasta",
                                                out_prefix=None, create_dir_for_each_family=False):
        from RouToolPa.Routines import SequenceRoutines, FileRoutines
        fam_id_list = IdList()
        fam_dict = SynDict()
        #print(pep_file)
        FileRoutines.safe_mkdir(output_dir)
        out_dir = FileRoutines.check_path(output_dir)
        create_directory_for_each_family = True if out_prefix else create_dir_for_each_family
        if families_id_file:
            fam_id_list.read(families_id_file)
        fam_dict.read(fam_file, split_values=True, values_separator=",")
        protein_dict = SeqIO.index_db("tmp.idx", pep_file, format=pep_format)

        for fam_id in fam_id_list if families_id_file else fam_dict:
            if fam_id in fam_dict:
                if create_directory_for_each_family:
                    fam_dir = "%s%s/" % (out_dir, fam_id)
                    FileRoutines.safe_mkdir(fam_dir)
                    out_file = "%s%s.pep" % (fam_dir, out_prefix if out_prefix else fam_id)
                else:
                    out_file = "%s/%s.pep" % (out_dir, out_prefix if out_prefix else fam_id)

                SeqIO.write(SequenceRoutines.record_by_id_generator(protein_dict, fam_dict[fam_id], verbose=True),
                            out_file, format=pep_format)
            else:
                print("%s was not found" % fam_id)

        os.remove("tmp.idx")

    @staticmethod
    def add_length_to_fam_file(fam_file, len_file, out_file, close_after_if_file_object=False):
        fam_dict = SynDict()
        fam_dict.read(fam_file, split_values=True, comments_prefix="#")
        len_dict = SynDict()
        len_dict.read(len_file, comments_prefix="#")

        out_fd = out_file if isinstance(out_file, file) else open(out_file, "r")

        for family in fam_dict:
            len_list = []
            for member in fam_dict[family]:
                len_list.append(None if member not in len_dict else len_dict[member])

            out_fd.write("%s\t%s\t%s\n" % (family, ",".join(fam_dict[family]), ",".join(len_list)))

        if close_after_if_file_object:
            out_fd.close()




#!/usr/bin/env python
__author__ = 'mahajrod'

from collections import OrderedDict
from RouToolPa.Routines.File import FileRoutines


class ProjectRoutines(FileRoutines):
    def __init__(self):
        FileRoutines.__init__(self)

    def initiate(self, workdir, project_name, species_list):
        """
        Structure of project folder:
            <workdir>/
                <project_name>/
                    <species1>
                        genome_denovo/
                            reads/
                                raw/
                                filtered/
                            fastqc/
                                raw/
                                filtered/
                            kmer/
                                raw/
                                filtered/
                            assemblies/
                            annotation/
                                protein_coding_genes/
                                repeats/
                                ncRNA/
                                    tRNA/
                                    rRNA/
                                    miRNA/
                                    other_ncRNA/
                            analysis/
                                orthologs/
                                families/
                                positive_selection/
                                specific_genes/
                        genome_ref_assisted/
                            reference/
                            reads/
                                raw/
                                filtered/
                            fastqc/
                                raw/
                                filtered/
                            kmer/
                                raw/
                                filtered/
                            SNPcall/
                            analysis/
                                population/
                        transcriptome/
                            reads/
                                raw/
                                filtered/
                            fastqc/
                                raw/
                                filtered/
                            kmer/
                                raw/
                                filtered/
                            assemblies/
                            annotation/
                            alignment/
                            analysis/
                            alignment/
                            expression/
                        analysis/

                    <species2>
                        ...
                    ...

                    <speciesN>
                        ...

        """
        species_dir_dict = {
                            "genome_denovo": {
                                "reads": {
                                    "raw": {},
                                    "filtered": {}, },
                                "fastqc": {
                                    "raw": {},
                                    "filtered": {}, },
                                "kmer": {
                                    "raw": {},
                                    "filtered": {}, },
                                "assemblies": {},
                                "annotation": {
                                    "protein_coding_genes": {},
                                    "repeats": {
                                        "repeatmasker": {},
                                        "TRF": {},
                                        "windowmasker": {}, },
                                    "ncRNA": {
                                        "tRNA": {},
                                        "rRNA": {},
                                        "miRNA": {},
                                        "other_ncRNA": {}, }, },
                                "analysis": {
                                    "orthologs": {},
                                    "families": {},
                                    "positive_selection": {},
                                    "specific_genes": {}, }
                                                },
                            "genome_ref_assisted": {
                                "reference": {},
                                "reads": {
                                    "raw": {},
                                    "filtered": {}, },
                                "fastqc": {
                                    "raw": {},
                                    "filtered": {}, },
                                "kmer": {
                                    "raw": {},
                                    "filtered": {}, },
                                "alignment": {},
                                "SNPcall": {
                                    "raw_vcf": {},
                                    "filtered_vcf": {},
                                    },
                                "analysis": {
                                    "population": {}, },
                                                    },
                            "transcriptome": {
                                "reads": {
                                    "raw": {},
                                    "filtered": {}, },
                                "fastqc": {
                                    "raw": {},
                                    "filtered": {}, },
                                "kmer": {
                                    "raw": {},
                                    "filtered": {}, },
                                "assemblies": {},
                                "annotation": {},
                                "alignment": {},
                                "analysis": {},
                                "expression": {},
                                              },
                            "analysis": {},
                            }

        manuscripts_dict = {
                            "plans": {},
                            "articles": {},
                            "drafts": {},
                             "figures": {},
                                            }

        project_dir_structure_dict = OrderedDict()
        project_dir_structure_dict[project_name] = OrderedDict()

        for species in [species_list] if isinstance(species_list, str) else species_list:
            project_dir_structure_dict[project_name][species] = species_dir_dict

        project_dir_structure_dict[project_name]["manuscripts"] = manuscripts_dict

        self.recursive_mkdir(project_dir_structure_dict, out_dir=workdir,
                             description_filename="DESCRIPTION", description_text=None,
                             readme_filename=None, readme_text=None)




#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool



class FastqDump(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "fastq-dump", path=path, max_threads=max_threads)

    def unpack(self, input_sra, output_dir, paired_input=True, retain_original_ids=True):

        options = " --outdir %s" % output_dir
        options += " --split-3" if paired_input else ""
        options += " --origfmt" if retain_original_ids else ""
        options += " %s" % input_sra

        self.execute(options=options)

    def parallel_unpack(self, input_dir, output_dir, sra_id_list=None, paired_input=True, retain_original_ids=True):

        sra_id_list = sra_id_list if sra_id_list else os.listdir(input_dir)

        common_options = " --split-3" if paired_input else ""
        common_options += " --origfmt" if retain_original_ids else ""

        options_list = []

        for sra_id in sra_id_list:
            out_dir = "%s/%s/" % (output_dir, sra_id)
            input_sra = "%s/%s/%s.sra" % (input_dir, sra_id, sra_id)

            self.safe_mkdir(out_dir)

            options = common_options
            options += " --outdir %s" % out_dir
            options += " %s" % input_sra

            options_list.append(options)

        self.parallel_execute(options_list=options_list)

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool



class MCMCTree(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "mcmctree", path=path, max_threads=max_threads)
        self.example = """
          seed = -1
       seqfile = mtCDNApri123.txt
      treefile = mtCDNApri.trees
       outfile = out

         ndata = 3
       seqtype = 0    * 0: nucleotides; 1:codons; 2:AAs
       usedata = 1    * 0: no data; 1:seq like; 2:normal approximation; 3:out.BV (in.BV)
         clock = 2    * 1: global clock; 2: independent rates; 3: correlated rates
       RootAge = '<1.0'  * safe constraint on root age, used if no fossil for root.

         model = 0    * 0:JC69, 1:K80, 2:F81, 3:F84, 4:HKY85, 5:TC92, 6: TN93, 7: GTR, 8: UNREST, 9:REVu, 10: UNSETu
         alpha = 0    * alpha for gamma rates at sites
         ncatG = 5    * No. categories in discrete gamma

     cleandata = 0    * remove sites with ambiguity data (1:yes, 0:no)?

       BDparas = 1 1 0.1   * birth, death, sampling
    kappa_gamma = 6 2      * gamma prior for kappa
    alpha_gamma = 1 1      * gamma prior for alpha

    rgene_gamma = 2 20    * gammaDir prior for rate for genes
    sigma2_gamma = 1 10   * gammaDir prior for sigma^2     (for clock=2 or 3)

      finetune = 1: .1 .1 .1 .1 .1 .1 * auto (0 or 1): times, musigma2, rates, mixing, paras, FossilErr

         print = 1
        burnin = 2000
      sampfreq = 2
       nsample = 20000
        """

    def write_example_ctl(self, example_file):
        with open(example_file, "w") as example_fd:
            example_fd.write(self.example)

    def print_example_ctl(self):
        print(self.example)


    @staticmethod
    def generate_ctl_file(seq_file, tree_file, out_file, ctl_file, seed=-1, num_of_partitions=1,
                          seq_type="nucleotides", use_data=1, clock="global", root_age=None,
                          model="HKY85", ncatG=5, alpha_for_gamma_rates_at_sites=0.5,
                          birth=1.0, death=1.0, sampling=0.1,
                          alpha_gamma_alpha=1.0, alpha_gamma_beta=1.0, kappa_gamma_alpha=6.0, kappa_gamma_beta=2.0,
                          rgene_gamma_alpha=2.0, rgene_gamma_beta=2.0, sigma2_gamma_alpha=1.0, sigma2_gamma_beta=10.0,
                          remove_ambiguity_sites=False,
                          auto_finetune=True, times=0.1, rates=0.1, mixing=0.1, paras=0.1, RateParas=0.1, FossilErr=0.1,
                          num_of_burning=2000, sampling_frequency=2, number_of_samples=20000):

        if seq_type == "nucleotides":
            sequence_type = 0
        elif seq_type == "codons":
            sequence_type = 1
        elif seq_type == "aminoacids":
            sequence_type = 2
        else:
            raise(ValueError, "Wrong sequence type was set")
            
        if clock == "global":
            clock_type = 1
        elif clock == "independent":
            clock_type = 2
        elif clock == "correlated":
            clock_type = 3
        else:
            raise(ValueError, "Wrong clock type was set")    

        if model == "JC69":
            model_type = 0
        elif model == "K80":
            model_type = 1
        elif model == "F81":
            model_type = 2
        elif model == "F84":
            model_type = 3
        elif model == "HKY85":
            model_type = 4
        #elif model == "TC92":
        #    model_type = 5
        #elif model == "TN93":
        #    model_type = 6
        #elif model == "GTR":
        #    model_type = 7
        #elif model == "UNREST":
        #    model_type = 8
        #elif model == "REVu":
        #    model_type = 9
        #elif model == "UNSETu":
        #    model_type = 10

        else:
            raise(ValueError, "Wrong model was set")       
            
        options = "seed = %i\n" % seed
        options += "seqfile = %s\n" % seq_file
        options += "treefile = %s\n" % tree_file
        options += "outfile = %s\n" % out_file

        options += "\n"

        options += "ndata = %i\n" % num_of_partitions
        options += "seqtype = %i * 0: nucleotides; 1:codons; 2:AAs\n" % sequence_type
        options += "usedata = %i * 0: no data; 1:seq like; 2:normal approximation; 3:out.BV (in.BV)\n" % use_data
        options += "clock = %i * 1: global clock; 2: independent rates; 3: correlated rates\n" % clock_type
        options += "RootAge = '%s' * safe constraint on root age, used if no fossil for root.\n" % root_age if root_age else ""

        options += "\n"

        options += "model = %i * 0:JC69, 1:K80, 2:F81, 3:F84, 4:HKY85\n" % model_type
        options += "alpha = %f * alpha for gamma rates at sites\n" % alpha_for_gamma_rates_at_sites
        options += "ncatG = %i * No. categories in discrete gamma\n" % ncatG

        options += "\n"

        options += "cleandata = %i * remove sites with ambiguity data (1:yes, 0:no)?\n" % (1 if remove_ambiguity_sites else 0)

        options += "\n"

        options += "BDparas = %f %f %f * birth, death, sampling\n" % (birth, death, sampling)
        options += "kappa_gamma = %f %f * gamma prior for kappa\n" % (kappa_gamma_alpha, kappa_gamma_beta)
        options += "alpha_gamma = %f %f * gamma prior for alpha\n" % (alpha_gamma_alpha, alpha_gamma_beta)

        options += "\n"

        """
        Choice of time unit. The time unit should be chosen such that the node ages are about 0.01-10.
        If the divergence times are around 100-1000MY, then 100MY may be one time unit. The priors
        on times and on rates and the fossil calibrations should all be specified based on your choice of
        the time scale. For example, if one time unit is 10MY, the following
        rgene_gamma = 2 20
        sigma2_gamma = 10 100
        * gamma prior for overall rates for genes
        * gamma prior for sigma^2 (for clock=2 or 3)
        means an overall average rate of 2/20 = 0.1 substitutions per site per 10MY or 8-10 substitutions
        per site per year, which is reasonable for mammalian mitochondrial genes. The gamma prior
        here has the shape parameter alpha = 2, and is fairly diffuse. If you change the time unit, you
        should keep the shape parameter fixed and change the scale parameter beta to have the correct
        mean. In other words, to use one time unit to represent 100MY, the above should become
        rgene_gamma = 2 2
        sigma2_gamma = 10 100
        * gamma prior for overall rates for genes
        * gamma prior for sigma^2 (for clock=2 or 3)
        Note that under the independent-rates model (clock=2), the change of the time unit should not
        lead to a change to the prior for sigma^2, because sigma^2 is the variance of the log rate: the variance of the
        logarithm of the rate does not change when you rescale the rate by a constant. However, for the
        correlated-rates model (clock=3), the change of the time unit should also lead to a change to sigma^2:
        under that model, the variance of the log-normal distribution is tsigma^2, where t is the time gap
        separating the midpoints of the branches.
        When you change the time unit, the fossil calibrations in the tree file should be changed
        accordingly. While ideally one would want the biological results to be unchanged when one
        changes the time unit, we know that two components of the model are not invariant to the time
        scale: the log normal distribution for rates and the birth-death model for times. Nevertheless,
        tests by Mathieu Groussin did suggest that the choice of time scale has very minimal effects on
        the posterior time and rate estimates.
        """
        options += "rgene_gamma = %f %f * gammaDir prior for rate for genes\n" % (rgene_gamma_alpha, rgene_gamma_beta)
        if clock_type > 1:
            options += "sigma2_gamma = %f %f * gammaDir prior for sigma^2     (for clock=2 or 3)\n" % \
                       (sigma2_gamma_alpha, sigma2_gamma_beta)

        options += "\n"

        options += "finetune = %i: %f %f %f %f %f %f  * auto (0 or 1) : times, rates, mixing, paras, RateParas, FossilErr\n" % \
                   (1 if auto_finetune else 0, times, rates, mixing, paras, RateParas, FossilErr)

        options += "\n"

        options += "print = 1 * write mcmc and summary to disk (mcmc.out)\n"
        """
        burnin, sampfreq and nsample: in our example, the program will discard thefirst 2,000 iterations as burn-in,
        and then it will sample every 2 iterations until it has gathered 20,000 samples. In total, the MCMC will run
        for 2,000 + 2 x 20, 000 = 42, 000 iterations. Normally, you should gather between 10,000 to 20,000 samples
        for a good statistical summary. Large sample sizes (say 100,000)tend to waste a lot of hard drive space
        providing very little statistical improvement. It may also take a long time for the program to summarize
        the results. If you need to increase the length of the MCMC (to improve convergence), increase sampfreq
        but keep nsample at a reasonable value.
        """
        options += "burnin = %i\n" % num_of_burning
        options += "sampfreq = %i\n" % sampling_frequency
        options += "nsample = %i\n" % number_of_samples

        with open(ctl_file, "w") as ctl_fd:
            ctl_fd.write(options)

    def run_mcmctree(self, ctl_file):

        options = " %s" % ctl_file

        self.execute(options=options)

    def run(self, seq_file, tree_file, out_file, ctl_file, seed=-1, num_of_partitions=1,
            seq_type="nucleotides", use_data=1, clock="global", root_age=None,
            model="HKY85", ncatG=5, alpha_for_gamma_rates_at_sites=0.5,
            birth=1.0, death=1.0, sampling=0.1,
            alpha_gamma_alpha=1.0, alpha_gamma_beta=1.0, kappa_gamma_alpha=6.0, kappa_gamma_beta=2.0,
            rgene_gamma_alpha=2.0, rgene_gamma_beta=2.0, sigma2_gamma_alpha=1.0, sigma2_gamma_beta=10.0,
            remove_ambiguity_sites=False,
            auto_finetune=True, times=0.1, rates=0.1, mixing=0.1, paras=0.1, RateParas=0.1, FossilErr=0.1,
            num_of_burning=2000, sampling_frequency=2, number_of_samples=20000):

        self.generate_ctl_file(seq_file, tree_file, out_file, ctl_file, seed=seed, num_of_partitions=num_of_partitions,
                               seq_type=seq_type, use_data=use_data, clock=clock, root_age=root_age,
                               model=model, ncatG=ncatG, alpha_for_gamma_rates_at_sites=alpha_for_gamma_rates_at_sites,
                               birth=birth, death=death, sampling=sampling,
                               alpha_gamma_alpha=alpha_gamma_alpha, alpha_gamma_beta=alpha_gamma_beta,
                               kappa_gamma_alpha=kappa_gamma_alpha, kappa_gamma_beta=kappa_gamma_beta,
                               rgene_gamma_alpha=rgene_gamma_alpha, rgene_gamma_beta=rgene_gamma_beta,
                               sigma2_gamma_alpha=sigma2_gamma_alpha, sigma2_gamma_beta=sigma2_gamma_beta,
                               remove_ambiguity_sites=remove_ambiguity_sites,
                               auto_finetune=auto_finetune, times=times, rates=rates, mixing=mixing, paras=paras,
                               RateParas=RateParas, FossilErr=FossilErr,
                               num_of_burning=num_of_burning, sampling_frequency=sampling_frequency,
                               number_of_samples=number_of_samples)

        options = " %s" % ctl_file

        self.execute(options=options)

    def run_all_clocks(self, seq_file, tree_file, output_directory, output_prefix="divtime", seed=-1, num_of_partitions=1,
                       seq_type="nucleotides", use_data=1,root_age=None,
                       model="HKY85", ncatG=5, alpha_for_gamma_rates_at_sites=0.5,
                       birth=1.0, death=1.0, sampling=0.1,
                       alpha_gamma_alpha=1.0, alpha_gamma_beta=1.0, kappa_gamma_alpha=6.0, kappa_gamma_beta=2.0,
                       rgene_gamma_alpha=2.0, rgene_gamma_beta=2.0, sigma2_gamma_alpha=1.0, sigma2_gamma_beta=10.0,
                       remove_ambiguity_sites=False,
                       auto_finetune=True, times=0.1, rates=0.1, mixing=0.1, paras=0.1, RateParas=0.1, FossilErr=0.1,
                       num_of_burning=2000, sampling_frequency=2, number_of_samples=20000,
                       cpus_per_task=1,
                       duplicate_to_stdout=False,
                       handling_mode="local",
                       job_name="divtime",
                       max_jobs=None,
                       max_running_time=None,
                       max_memory_per_node=None,
                       max_memmory_per_cpu=None,
                       modules_list=None,
                       environment_variables_dict=None):

        self.safe_mkdir(output_directory)

        prefix = self.get_basename(output_prefix)

        dir_list = []
        options_list = []
        output_prefix_list = []
        stdout_file_list = []

        cmd_log_file = "%s/%s.cmd" % (output_directory, prefix)
        log_prefix = "%s/%s" % (output_directory, prefix)
        error_log_prefix = "%s/%s" % (output_directory, prefix)

        for clock in "global", "independent", "correlated":
            out_dir = os.path.abspath("%s/%s/" % (output_directory, clock))

            self.safe_mkdir(out_dir)

            for filename in tree_file, seq_file:
                os.system("ln %s %s " % (filename, out_dir))

            out_file = "%s%s.out" % ("%s." % prefix if prefix else "", clock)
            out_pref = "%s/%s%s" % (out_dir, "%s." % prefix if prefix else "", clock)
            ctl_file = "%s.ctl" % out_pref

            self.generate_ctl_file(self.get_basename(seq_file), self.get_basename(tree_file),
                                   out_file, ctl_file,
                                   seed=seed, num_of_partitions=num_of_partitions,
                                   seq_type=seq_type, use_data=use_data, clock=clock, root_age=root_age,
                                   model=model, ncatG=ncatG,
                                   alpha_for_gamma_rates_at_sites=alpha_for_gamma_rates_at_sites,
                                   birth=birth, death=death, sampling=sampling,
                                   alpha_gamma_alpha=alpha_gamma_alpha, alpha_gamma_beta=alpha_gamma_beta,
                                   kappa_gamma_alpha=kappa_gamma_alpha, kappa_gamma_beta=kappa_gamma_beta,
                                   rgene_gamma_alpha=rgene_gamma_alpha, rgene_gamma_beta=rgene_gamma_beta,
                                   sigma2_gamma_alpha=sigma2_gamma_alpha, sigma2_gamma_beta=sigma2_gamma_beta,
                                   remove_ambiguity_sites=remove_ambiguity_sites,
                                   auto_finetune=auto_finetune, times=times, rates=rates, mixing=mixing, paras=paras,
                                   RateParas=RateParas, FossilErr=FossilErr,
                                   num_of_burning=num_of_burning, sampling_frequency=sampling_frequency,
                                   number_of_samples=number_of_samples)
            dir_list.append(out_dir)
            output_prefix_list.append(out_pref)
            options_list.append(" %s" % ctl_file)
            stdout_file_list.append("%s%s.stdout" % ("%s." % prefix if prefix else "", clock))

        if handling_mode == "local":
            self.parallel_execute(options_list, dir_list=dir_list, threads=3, output_file_list=stdout_file_list,
                                  duplicate_to_stdout=duplicate_to_stdout)
        elif handling_mode == "slurm":
            cmd_list = []
            for options, directory, output_pref in zip(options_list, dir_list, output_prefix_list):

                cmd = " cd %s; " % directory
                cmd += "%s%s %s > %s.stdout 2>&1" % ((self.path + "/") if self.path else "",
                                                     self.cmd, options, output_pref)
                cmd_list.append(cmd)

            self.slurm_run_multiple_jobs_in_wrap_mode(cmd_list,
                                                      cmd_log_file,
                                                      max_jobs=max_jobs,
                                                      job_name=job_name,
                                                      log_prefix=log_prefix,
                                                      error_log_prefix=error_log_prefix,
                                                      cpus_per_node=None,
                                                      max_running_jobs=None,
                                                      max_running_time=max_running_time,
                                                      cpus_per_task=cpus_per_task,
                                                      max_memory_per_node=max_memory_per_node,
                                                      max_memmory_per_cpu=max_memmory_per_cpu,
                                                      modules_list=modules_list,
                                                      environment_variables_dict=environment_variables_dict)


#!/usr/bin/env python
__author__ = 'mahajrod'

import pyparsing as pyp


class TrimmomaticReport:
    def __init__(self, trimmomatic_report, input_is_se=False):

        if input_is_se:
            input = pyp.Literal("Input Reads: ")
            input_number = pyp.Word(pyp.nums)
            surv = pyp.Literal("Surviving: ")
            surv_number = pyp.Word(pyp.nums)

            str1 = pyp.Literal("(")
            surv_percent = pyp.Word(pyp.nums + ".-")
            dropped = pyp.Literal("%) Dropped: ")
            dropped_number = pyp.Word(pyp.nums)
            dropped_percent = pyp.Word(pyp.nums + ".-")
            end = pyp.Literal("%)")

            for token in surv_number, dropped_number, input_number:
                token.setParseAction(lambda s, l, t: [int(t[0])])
            for token in surv_percent, dropped_percent:
                token.setParseAction(lambda s, l, t: [float(t[0])])

            pattern = input + input_number + surv + surv_number + str1 + surv_percent + dropped + dropped_number + \
                      str1 + dropped_percent + end

            with open(trimmomatic_report, "r") as log_fd:
                for line in log_fd:
                    if line[:12] == "Input Reads:":
                        parsed_list = pattern.parseString(line.strip())

            self.stats = {
                          "input":                   parsed_list[1],
                          "surviving":               parsed_list[3],
                          "surviving,%":             parsed_list[5],
                          "dropped":                 parsed_list[7],
                          "dropped,%":               parsed_list[9]
                         }

        else:

            input = pyp.Literal("Input Read Pairs: ")
            input_number = pyp.Word(pyp.nums)
            both = pyp.Literal("Both Surviving: ")
            both_number = pyp.Word(pyp.nums)

            str1 = pyp.Literal("(")
            both_percent = pyp.Word(pyp.nums + ".-")
            forward = pyp.Literal("%) Forward Only Surviving: ")
            forward_number = pyp.Word(pyp.nums)
            forward_percent = pyp.Word(pyp.nums + ".-")
            reverse = pyp.Literal("%) Reverse Only Surviving: ")
            reverse_number = pyp.Word(pyp.nums)
            reverse_percent = pyp.Word(pyp.nums + ".-")
            dropped = pyp.Literal("%) Dropped: ")
            dropped_number = pyp.Word(pyp.nums)
            dropped_percent = pyp.Word(pyp.nums + ".-")
            end = pyp.Literal("%)")

            for token in both_number, forward_number, reverse_number, dropped_number, input_number:
                token.setParseAction(lambda s, l, t: [int(t[0])])
            for token in both_percent, forward_percent, reverse_percent, dropped_percent:
                token.setParseAction(lambda s, l, t: [float(t[0])])

            both_number.setParseAction(lambda s, l, t: [int(t[0])])

            pattern = input + input_number + both + both_number + str1 + both_percent + forward + forward_number + str1 + \
                      forward_percent + reverse + reverse_number + str1 + reverse_percent + dropped + dropped_number + \
                      str1 + dropped_percent + end

            with open(trimmomatic_report, "r") as log_fd:
                for line in log_fd:
                    if line[:17] == "Input Read Pairs:":
                        parsed_list = pattern.parseString(line.strip())

            self.stats = {
                          "input":                     parsed_list[1],
                          "both_surviving":            parsed_list[3],
                          "both_surviving,%":          parsed_list[5],
                          "forward_only_surviving":    parsed_list[7],
                          "forward_only_surviving,%":  parsed_list[9],
                          "reverse_only_surviving":    parsed_list[11],
                          "reverse_only_surviving,%":  parsed_list[13],
                          "dropped":                   parsed_list[15],
                          "dropped,%":                 parsed_list[17]
                         }





#!/usr/bin/env python
"""
Last output parser Module based on pandas
"""
__author__ = 'Sergei F. Kliver'
import datetime
from copy import deepcopy
from collections import OrderedDict

import numpy as np
import pandas as pd

import RouToolPa.Formats.AlignmentFormats as AlignmentFormats


class CollectionBLAST:

    def __init__(self, in_file=None, records=None,
                 format="tab6", parsing_mode="complete",
                 target_black_list=(), target_white_list=(),
                 query_black_list=(), query_white_list=(),
                 target_syn_dict=None, query_syn_dict=None,
                 min_target_hit_len=None, min_query_hit_len=None,
                 min_target_len=None, min_query_len=None):

        self.formats = ["tab6", "busco_tblastn"]
        self.parsing_parameters = {"tab6": {"complete": {
                                                   "col_names": ["query_id",
                                                                 "target_id",
                                                                 "identity,%%",
                                                                 "length",
                                                                 "mismatch",
                                                                 "gapopen",
                                                                 "query_start",
                                                                 "query_end",
                                                                 "target_start",
                                                                 "target_end",
                                                                 "evalue",
                                                                 "bitscore"],
                                                   "cols":      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                                                   "index_cols": ["query_id", "target_id"],
                                                   "converters": {
                                                                  "qseqid":     str,
                                                                  "sseqid":     str,
                                                                  "pident":     np.float32,
                                                                  "length":     np.int64,
                                                                  "mismatch":   np.int32,
                                                                  "gapopen":    np.int32,
                                                                  "qstart":     np.int64,
                                                                  "qend":       np.int64,
                                                                  "sstart":     np.int64,
                                                                  "send":       np.int64,
                                                                  "evalue":     np.float64,
                                                                  "bitscore":   np.float32
                                                                 },
                                                   "col_name_indexes": {
                                                                        "query_id":         0,
                                                                        "target_id":         1,
                                                                        "identity,%%":      2,
                                                                        "length":           3,
                                                                        "mismatch":         4,
                                                                        "gapopen":          5,
                                                                        "query_start":      6,
                                                                        "query_end":        7,
                                                                        "target_start":     8,
                                                                        "target_end":       9,
                                                                        "evalue":           10,
                                                                        "bitscore":         11
                                                                        },
                                                   "original_col_names": ["qseqid",
                                                                          "sseqid",
                                                                          "pident",
                                                                          "length",
                                                                          "mismatch",
                                                                          "gapopen",
                                                                          "qstart",
                                                                          "qend",
                                                                          "sstart",
                                                                          "send",
                                                                          "evalue",
                                                                          "bitscore"],
                                                   "original_col_name_indexes": {
                                                                                 "qseqid":     0,
                                                                                 "sseqid":     1,
                                                                                 "pident":     2,
                                                                                 "length":     3,
                                                                                 "mismatch":   4,
                                                                                 "gapopen":    5,
                                                                                 "qstart":     6,
                                                                                 "qend":       7,
                                                                                 "sstart":     8,
                                                                                 "send":       9,
                                                                                 "evalue":     10,
                                                                                 "bitscore":   11
                                                                                 },
                                                   },

                                           },
                                 "busco_tblastn": {"complete": {
                                                               "col_names": ["query_id",
                                                                             "target_id",
                                                                             "identity,%%",
                                                                             "length",
                                                                             "mismatch",
                                                                             "gapopen",
                                                                             "query_start",
                                                                             "query_end",
                                                                             "target_start",
                                                                             "target_end",
                                                                             "evalue",
                                                                             "bitscore"],
                                                               "cols":      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                                                               "index_cols": ["query_id", "target_id"],
                                                               "converters": {
                                                                              "qseqid":     str,
                                                                              "sseqid":     str,
                                                                              "pident":     np.float32,
                                                                              "length":     np.int64,
                                                                              "mismatch":   np.int32,
                                                                              "gapopen":    np.int32,
                                                                              "qstart":     np.int64,
                                                                              "qend":       np.int64,
                                                                              "sstart":     np.int64,
                                                                              "send":       np.int64,
                                                                              "evalue":     np.float64,
                                                                              "bitscore":   np.float32
                                                                             },
                                                               "col_name_indexes": {
                                                                                    "query_id":         0,
                                                                                    "target_id":         1,
                                                                                    "identity,%%":      2,
                                                                                    "length":           3,
                                                                                    "mismatch":         4,
                                                                                    "gapopen":          5,
                                                                                    "query_start":      6,
                                                                                    "query_end":        7,
                                                                                    "target_start":     8,
                                                                                    "target_end":       9,
                                                                                    "evalue":           10,
                                                                                    "bitscore":         11
                                                                                    },
                                                               "original_col_names": ["qseqid",
                                                                                      "sseqid",
                                                                                      "pident",
                                                                                      "length",
                                                                                      "mismatch",
                                                                                      "gapopen",
                                                                                      "qstart",
                                                                                      "qend",
                                                                                      "sstart",
                                                                                      "send",
                                                                                      "evalue",
                                                                                      "bitscore"],
                                                               "original_col_name_indexes": {
                                                                                             "qseqid":     0,
                                                                                             "sseqid":     1,
                                                                                             "pident":     2,
                                                                                             "length":     3,
                                                                                             "mismatch":   4,
                                                                                             "gapopen":    5,
                                                                                             "qstart":     6,
                                                                                             "qend":       7,
                                                                                             "sstart":     8,
                                                                                             "send":       9,
                                                                                             "evalue":     10,
                                                                                             "bitscore":   11
                                                                                             },
                                                                },

                                           },
                                    }
        self.parsing_mode = parsing_mode

        self.format = format
        self.target_black_list = target_black_list
        self.target_white_list = target_white_list
        self.target_syn_dict = target_syn_dict

        self.query_black_list = query_black_list
        self.query_white_list = query_white_list
        self.query_syn_dict = query_syn_dict
        # attributes type conversion parameters

        self.converters = OrderedDict()
        self.pandas_int_type_correspondence = OrderedDict({
                                                           "Int8":  np.float16,
                                                           "Int16": np.float16,
                                                           "Int32": np.float32,
                                                           "Int64": np.float64,
                                                           })

        # init aliases
        self.col_names = self.parsing_parameters[self.format][self.parsing_mode]["col_names"]
        self.index_cols = self.parsing_parameters[self.format][self.parsing_mode]["index_cols"]
        self.target_id_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_id"]
        self.target_start_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_start"]
        #self.target_hit_len_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_hit_len"]
        #self.target_strand_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_strand"]

        self.query_id_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_id"]
        self.query_start_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_start"]
        #self.query_hit_len_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_hit_len"]
        #self.query_strand_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_strand"]

        #
        self.target_scaffold_list = None
        self.query_scaffold_list = None
        self.target_scaffold_lengths = None
        self.query_scaffold_lengths = None

        if in_file:
            self.read(in_file,
                      format=format,
                      parsing_mode=parsing_mode,
                      target_black_list=target_black_list,
                      target_white_list=target_white_list,
                      query_black_list=query_black_list,
                      query_white_list=query_white_list,
                      min_target_hit_len=min_target_hit_len,
                      min_query_hit_len=min_query_hit_len,
                      min_target_len=min_target_len,
                      min_query_len=min_query_len)

        else:
            self.records = records

    def read(self, in_file,
             format="tab6", parsing_mode="all",
             target_black_list=(), target_white_list=(),
             query_black_list=(), query_white_list=(),
             min_target_hit_len=None, min_query_hit_len=None,
             min_target_len=None, min_query_len=None):
        if format not in self.parsing_parameters:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing!" % parsing_mode)
        elif parsing_mode not in self.parsing_parameters[format]:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing in this mode(%s)!" % (format, parsing_mode))

        print("%s\tReading input..." % str(datetime.datetime.now()))
        self.records = pd.read_csv(in_file, sep='\t', header=None, na_values=".",
                                   comment="#",
                                   usecols=self.parsing_parameters[format][parsing_mode]["cols"],
                                   converters=self.parsing_parameters[format][parsing_mode]["converters"],
                                   names=self.parsing_parameters[format][parsing_mode]["col_names"])


        print("%s\tReading input finished..." % str(datetime.datetime.now()))
        print("%s\tFiltering..." % str(datetime.datetime.now()))
        if target_white_list or target_black_list:
            target_scaffolds_to_keep = self.get_filtered_entry_list(self.records["target_id"].tolist(),
                                                                    entry_black_list=target_black_list,
                                                                    entry_white_list=target_white_list)
            #print target_scaffolds_to_keep
            self.records = self.records[self.records["target_id"].isin(target_scaffolds_to_keep)]

        if query_white_list or query_black_list:
            query_scaffolds_to_keep = self.get_filtered_entry_list(self.records["query_id"].tolist(),
                                                                   entry_black_list=query_black_list,
                                                                   entry_white_list=query_white_list)
            #print query_scaffolds_to_keep
            self.records = self.records[self.records["query_id"].isin(query_scaffolds_to_keep)]
        if self.target_syn_dict:
            self.records["target_id"].replace(self.target_syn_dict, inplace=True)
        if self.query_syn_dict:
            self.records["query_id"].replace(self.query_syn_dict, inplace=True)
        # retain only automatically generated index by row number
        #self.records.index = pd.MultiIndex.from_arrays([self.records.index, np.arange(0, len(self.records))],
        #                                               names=("scaffold", "row"))
        print("%s\tFiltering finished..." % str(datetime.datetime.now()))

        if min_target_len and min_query_len:
            self.records = self.records[(self.records["target_len"] >= min_target_len) & (self.records["query_len"] >= min_query_len)]
        elif min_target_len:
            self.records = self.records[self.records["target_len"] >= min_target_len]
        elif min_query_len:
            self.records = self.records[self.records["query_len"] >= min_query_len]
            
        if min_target_hit_len and min_query_hit_len:
            self.records = self.records[(self.records["target_hit_len"] >= min_target_hit_len) & (self.records["query_hit_len"] >= min_query_hit_len)]
        elif min_target_hit_len:
            self.records = self.records[self.records["target_hit_len"] >= min_target_hit_len]
        elif min_query_hit_len:
            self.records = self.records[self.records["query_hit_len"] >= min_query_hit_len]

        if "target_len" in self.parsing_parameters[self.format][self.parsing_mode]["col_names"]:
            self.target_scaffold_lengths = self.records[["target_id", "target_len"]].drop_duplicates()
            self.target_scaffold_lengths.columns = ["id", "length"]
            self.target_scaffold_lengths.set_index("id", inplace=True)
            self.target_scaffold_list = self.target_scaffold_lengths.index.tolist()
        if "query_len" in self.parsing_parameters[self.format][self.parsing_mode]["col_names"]:
            self.query_scaffold_lengths = self.records[["query_id", "query_len"]].drop_duplicates()
            self.query_scaffold_lengths.columns = ["id", "length"]
            self.query_scaffold_lengths.set_index("id", inplace=True)
            self.query_scaffold_list = self.query_scaffold_lengths.index.tolist()

        self.records.set_index(self.parsing_parameters[format][parsing_mode]["index_cols"], inplace=True)

        """
        retained_columns = deepcopy(self.parsing_parameters[self.format][self.parsing_mode]["col_names"])
        for entry in "target_len", "query_len":
            if entry in retained_columns:
                retained_columns.remove(entry)

        self.records = self.records[retained_columns]
        """
    """
    def sort(self, inplace=False,
             sorting_order=("target_id", "query_id", "target_start", "target_hit_len", "query_start", "query_hit_len")):
        return self.records.sort_values(by=sorting_order, inplace=inplace)
    """

    @staticmethod
    def get_filtered_entry_list(entry_list,
                                entry_black_list=[],
                                sort_entries=False,
                                entry_ordered_list=None,
                                entry_white_list=[]):
        white_set = set(entry_white_list)
        black_set = set(entry_black_list)
        entry_set = set(entry_list)

        if white_set:
            entry_set = entry_set & white_set
        if black_set:
            entry_set = entry_set - black_set

        filtered_entry_list = list(entry_set)
        if sort_entries:
            filtered_entry_list.sort()

        final_entry_list = []

        if entry_ordered_list:
            for entry in entry_ordered_list:
                if entry in filtered_entry_list:
                    final_entry_list.append(entry)
                    filtered_entry_list.remove(entry)
                else:
                    print("WARNING!!!Entry(%s) from order list is absent in list of entries!" % entry)
            return final_entry_list + filtered_entry_list
        else:
            return filtered_entry_list

    def write(self, output, separator="\t", header=False):
        self.records.to_csv(output,
                            sep=separator,
                            header=header,
                            index=False)

    def rename_target_ids(self, syn_dict):
        self.records[["target_id"]].replace(syn_dict, inplace=True)

    def rename_query_ids(self, syn_dict):
        self.records[["query_id"]].replace(syn_dict, inplace=True)




#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class FastQC(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "fastqc", path=path, max_threads=max_threads)

    def check(self, input_files, out_dir="./", kmer_size=10, nogroup=True, noextract=False, adapters_file=None):
        options = " -o %s" % out_dir
        options += " --nogroup" if nogroup else ""
        options += " --noextract" if noextract else ""
        options += " -t %i" % self.threads
        options += " -a %s" % adapters_file if adapters_file else ""
        options += " -k %i" % kmer_size
        options += " %s" % input_files if isinstance(input_files, str) else " ".join(input_files)
        self.execute(options, cmd="fastqc")


if __name__ == "__main__":
    pass



from RouToolPa.Tools.Abstract import Tool


class BWA(Tool):
    def __init__(self, path="", max_threads=4, max_memory="100G", max_per_thread_memory="5G"):
        Tool.__init__(self, "bwa", path=path, max_threads=max_threads, max_memory=max_memory, max_per_thread_memory=max_per_thread_memory)

    def index(self, fasta_file, prefix=None):
        index_prefix = ""
        if prefix:
            index_prefix = "-p %s" % prefix
        options = "%s %s" % (index_prefix, fasta_file)
        self.execute(options, cmd="bwa index")
        #os.system("bwa index %s %s" % (index_prefix, fasta_file))

    def mem(self, index, forward_reads, reverse_reads=None, output_file="alignment.sam"):
        reads = forward_reads
        if reverse_reads:
            reads += " %s" % reverse_reads
        options = "-t %i %s %s > %s" % (self.threads, index, reads, output_file)
        self.execute(options, cmd="bwa mem")
        #os.system("bwa mem -t %i %s %s > %s" % (max_threads, index, reads, output_file))

    def align(self, index,
              forward_reads_list=None,
              reverse_reads_list=None,
              unpaired_reads_list=None,
              quality_score="phred33",
              output_prefix="alignment",
              output_format="bam",
              read_group_name="reads",
              PU="x",
              SM="sample",
              platform="Illumina",
              LB="x",
              sort_by_coordinate=True,
              sort_by_name=False,
              max_per_sorting_thread_memory=None
              ):
        """
        Common method for all aligners
        """

        if (not forward_reads_list) and (not reverse_reads_list):
            raise ValueError("No read files were given")

        reads = forward_reads_list if isinstance(forward_reads_list, str) else  forward_reads_list[0]
        if reverse_reads_list:
            reads += " %s" % reverse_reads_list if isinstance(reverse_reads_list, str) else reverse_reads_list[0]

        options = " -t %i" % self.threads
        options += " %s" % index
        options += " %s " % reads
        options += " -R \'@RG\\tID:%s\\tPU:%s\\tSM:%s\\tPL:%s\\tLB:%s\'" % (read_group_name, PU, SM, platform, LB)

        if sort_by_coordinate or sort_by_name:
            if sort_by_coordinate and sort_by_name:
                raise ValueError("Sorting by both coordinate and read name was requested")
            options += " | samtools sort"
            if sort_by_name:
                options += " -n"
            options += " -@ %i" % self.threads
            options += " -m %s" % max_per_sorting_thread_memory if max_per_sorting_thread_memory else self.max_per_thread_memory
            options += " -O %s" % output_format.upper()

        options += " > %s.%s" % (output_prefix, output_format)
        self.execute(options, cmd="bwa mem")

    def aln(self, in_db_fasta, in_query_fastq, out_sai, max_edit_dist=0.04, max_gaps=1, max_gap_ext=-1, disallow_lond_del_limit=16,
            disallow_indel_limit=5, max_seed_edit_dist=2, mismatch_penalty=3, gap_open_penalty=11,
            gap_extension_penalty=4, trim=0, barcode_length=0):

        options = "-t %i -n %f -o %i -e %i -d %i -i %s -k %i -M %i -O %i -E %i -q %i -B %i" %\
                  (self.threads, max_edit_dist, max_gaps, max_gap_ext, disallow_lond_del_limit, disallow_indel_limit,
                   max_seed_edit_dist, mismatch_penalty, gap_open_penalty, gap_extension_penalty, trim, barcode_length)

        options += " %s %s %s" % (in_db_fasta, in_query_fastq, out_sai)

        self.execute(options, cmd="bwa aln")


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class DiscovarDeNovo(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "DiscovarDeNovo", path=path, max_threads=max_threads)

    def get_stat(self, list_of_read_files, output_dir, reference):

        options = " NUM_THREADS=%i" % self.threads
        options += " READS=%s" % ",".join(list_of_read_files)
        options += " OUT_DIR=%s" % output_dir
        options += " REFHEAD=%s" % reference

        self.execute(options)

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class NextClip(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "nextclip", path=path, max_threads=max_threads)

    def filter(self, forward_reads, reverse_reads, output_prefix, log_file, min_len=25, number_of_reads=None,
               trim_ends=0, remove_duplicates=True, use_category_e=False, duplicates_log=False, strict_match=None,
               relaxed_match=None,  adaptor_sequence=None):
        # strict_match and relaxed_match have to be tuples with 2 numbers
        # Default values:
        # [-x | --strict_match] Strict alignment matches (default '34,18')
        # [-y | --relaxed_match] Relaxed alignment matches (default '32,17')

        options = " --input_one %s" % forward_reads
        options += " --input_two %s" % reverse_reads
        options += " --output_prefix %s" % output_prefix
        options += " --log %s" % log_file
        options += " --min_length %i" % min_len
        options += " --number_of_reads %i" % number_of_reads if number_of_reads else ""
        options += " --trim_ends %i" % trim_ends
        options += " --remove_duplicates" if remove_duplicates else ""
        options += " --use_category_e" if use_category_e else ""
        options += " --duplicates_log %s" % duplicates_log if duplicates_log else ""
        options += " --strict_match %i,%i" % (strict_match[0], strict_match[1]) if strict_match else ""
        options += " --relaxed_match %i,%i" % (relaxed_match[0], relaxed_match[1]) if relaxed_match else ""
        options += " --adaptor_sequence %s" % adaptor_sequence if adaptor_sequence else ""

        self.execute(options, cmd="filter_by_mean_quality")


if __name__ == "__main__":
    pass






import os
from RouToolPa.Tools.Abstract import JavaTool



class SelectVariants(JavaTool):

    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T SelectVariants", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_variantutils_SelectVariants.html
    # selectType
    # INDEL
    # SNP
    # MIXED
    # MNP
    # SYMBOLIC
    # NO_VARIATION

    def select_variants(self, reference_file, input_vcf, output_vcf, vartype=None, varfilter=None):

        options = " -R %s" % reference_file
        options += " -V %s" % input_vcf
        options += " -selectType \'%s\'" % vartype if vartype else ""
        options += " -select \'%s\'" % varfilter if varfilter else ""
        options += " -o %s" % output_vcf

        self.execute(options=options)

    def get_SNP(self, reference_file, input_vcf, output_vcf):
        self.select_variants(reference_file, input_vcf, output_vcf, vartype="SNP")

    def get_indel(self, reference_file, input_vcf, output_vcf):
        self.select_variants(reference_file, input_vcf, output_vcf, vartype="INDEL")

    def remove_entries_with_filters(self, reference_file, input_vcf, output_vcf):
        options = " -R %s" % reference_file
        options += " -V %s" % input_vcf
        options += " -ef"
        options += " -o %s" % output_vcf
        self.execute(options=options)

    def get_STR(self, reference_file, input_vcf, output_vcf):
        #Use STR filtration based on GATK prediction very carefully as it counts even AAA -> AA as STR
        self.select_variants(reference_file, input_vcf, output_vcf, varfilter="STR")

    def get_nonSTR(self, reference_file, input_vcf, output_vcf):
        self.select_variants(reference_file, input_vcf, output_vcf, varfilter="vc.hasAttribute(\"STR\") == 0")




#!/usr/bin/env python
"""
GFF Parser Module based on pandas
"""
__author__ = 'Sergei F. Kliver'
import datetime
from copy import deepcopy
from collections import OrderedDict, Iterable
import numpy as np
import pandas as pd

from RouToolPa.Parsers.Abstract import Parser
import RouToolPa.Formats.AnnotationFormats as AnnotationFormats


class BUSCOtable(Parser):

    def __init__(self, in_file=None, records=None, format="busco_table", parsing_mode="complete",
                 black_list=(), white_list=(), add_row_index=True):

        self.formats = ["busco_table"]

        self.status_list = ["Complete",
                            "Duplicated",
                            "Fragmented",
                            "Missing"]

        self.parsing_parameters = {"busco_table": {

                                           "complete": {
                                                   "col_names": ["id", "status", "scaffold", "start", "end",
                                                                 "score", "length"],
                                                   "cols":      [0, 1, 2, 3, 4, 5, 6],
                                                   "index_cols": ["status"],
                                                   "converters": {
                                                                  "id":             str,
                                                                  "status":         str,
                                                                  "scaffold":       str,
                                                                  "start":          str, #lambda x: np.int32(x) - 1,
                                                                  "end":            str, #np.int32,
                                                                  "score":          str, #float,
                                                                  "length":         str, #np.int32,
                                                                  },
                                                   "col_name_indexes": {
                                                                        "id":           0,
                                                                        "status":       1,
                                                                        "scaffold":     2,
                                                                        "start":        3,
                                                                        "end":          4,
                                                                        "score":        5,
                                                                        "length":       6,
                                                                        },
                                                   },
                                           },
                                    }

        self.parsing_mode = parsing_mode
        self.format = format

        self.black_list = black_list
        self.white_list = white_list

        # init aliases
        self.record_id_col = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["id"]
        self.record_start_col = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["start"]
        self.record_scaffold_col = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["scaffold"]
        self.record_end_col = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["end"]
        self.record_status_col = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["status"]

        self.col_names = self.parsing_parameters[self.format][self.parsing_mode]["col_names"]
        self.index_cols = self.parsing_parameters[self.format][self.parsing_mode]["index_cols"]

        # load records
        if in_file:
            self.read(in_file, format=format, parsing_mode=parsing_mode, black_list=black_list, white_list=white_list,
                      add_row_index=add_row_index)

        else:
            self.records = records

    def read(self, in_file, format="busco_table", parsing_mode="only_coordinates", sort=False,
             black_list=(), white_list=(), add_row_index=True):
        if format not in self.parsing_parameters:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing!" % parsing_mode)
        elif parsing_mode not in self.parsing_parameters[format]:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing in this mode(%s)!" % (format, parsing_mode))

        print("%s\tReading input..." % str(datetime.datetime.now()))
        self.records = pd.read_csv(in_file, sep='\t', header=None, na_values=".",
                                   comment="#",
                                   usecols=self.parsing_parameters[format][parsing_mode]["cols"],
                                   converters=self.parsing_parameters[format][parsing_mode]["converters"],
                                   names=self.parsing_parameters[format][parsing_mode]["col_names"],
                                   index_col=self.parsing_parameters[format][parsing_mode]["index_cols"])

        if white_list or black_list:
            scaffolds_to_keep = self.get_filtered_entry_list(self.records.index, entry_black_list=black_list,
                                                             entry_white_list=white_list)
            self.records = self.records[self.records.index.get_level_values('scaffold').isin(scaffolds_to_keep)]

        if add_row_index:
            self.records.index = pd.MultiIndex.from_arrays([self.records.index, np.arange(0, len(self.records))],
                                                           names=("scaffold", "row"))
        print("%s\tReading input finished..." % str(datetime.datetime.now()))

        if sort:
            self.records.sort_values(by=["scaffold", "start", "end"])

    def count_statuses(self):
        status_dict = OrderedDict()
        count_dict = OrderedDict()
        for status in self.status_list:
            status_dict[status] = set(self.records.loc[status]["id"].unique())
            count_dict[status] = len(status_dict[status])

        return status_dict, count_dict
